@@ -60,20 +60,14 @@
 static const char* ep;
 #define FMT_WHITESPACE_CHAR ' '
 
-const char* cJSON_GetErrorPtr()
-{
-    return ep;
-}
+const char* cJSON_GetErrorPtr() { return ep; }
 
 static int cJSON_strcasecmp(const char* s1, const char* s2)
 {
-    if(!s1)
-        return (s1 == s2) ? 0 : 1;
-    if(!s2)
-        return 1;
+    if(!s1) return (s1 == s2) ? 0 : 1;
+    if(!s2) return 1;
     for(; tolower(*s1) == tolower(*s2); ++s1, ++s2)
-        if(*s1 == 0)
-            return 0;
+        if(*s1 == 0) return 0;
     return tolower(*(const unsigned char*)s1) - tolower(*(const unsigned char*)s2);
 }
 
@@ -86,8 +80,7 @@ static char* cJSON_strdup(const char* str)
     char* copy;
 
     len = strlen(str) + 1;
-    if(!(copy = (char*)cJSON_malloc(len)))
-        return 0;
+    if(!(copy = (char*)cJSON_malloc(len))) return 0;
     memcpy(copy, str, len);
     return copy;
 }
@@ -108,8 +101,7 @@ void cJSON_InitHooks(cJSON_Hooks* hooks)
 static cJSON* cJSON_New_Item()
 {
     cJSON* node = (cJSON*)cJSON_malloc(sizeof(cJSON));
-    if(node)
-        memset(node, 0, sizeof(cJSON));
+    if(node) memset(node, 0, sizeof(cJSON));
     return node;
 }
 
@@ -119,12 +111,9 @@ void cJSON_Delete(cJSON* c)
     cJSON* next;
     while(c) {
         next = c->next;
-        if(!(c->type & cJSON_IsReference) && c->child)
-            cJSON_Delete(c->child);
-        if(!(c->type & cJSON_IsReference) && c->valuestring)
-            cJSON_free(c->valuestring);
-        if(c->string)
-            cJSON_free(c->string);
+        if(!(c->type & cJSON_IsReference) && c->child) cJSON_Delete(c->child);
+        if(!(c->type & cJSON_IsReference) && c->valuestring) cJSON_free(c->valuestring);
+        if(c->string) cJSON_free(c->string);
         cJSON_free(c);
         c = next;
     }
@@ -137,12 +126,9 @@ static const char* parse_number(cJSON* item, const char* num)
     int subscale = 0, signsubscale = 1;
 
     /* Could use sscanf for this? */
-    if(*num == '-')
-        sign = -1, num++; /* Has sign? */
-    if(*num == '0')
-        num++; /* is zero */
-    if(*num >= '1' && *num <= '9')
-        do
+    if(*num == '-') sign = -1, num++; /* Has sign? */
+    if(*num == '0') num++;            /* is zero */
+    if(*num >= '1' && *num <= '9') do
             n = (n * 10.0) + (*num++ - '0');
         while(*num >= '0' && *num <= '9'); /* Number? */
     if(*num == '.' && num[1] >= '0' && num[1] <= '9') {
@@ -177,8 +163,7 @@ static char* print_number(cJSON* item)
     double d = item->valuedouble;
     if(fabs(((double)item->valueint) - d) <= DBL_EPSILON && d <= INT_MAX && d >= INT_MIN) {
         str = (char*)cJSON_malloc(21); /* 2^64+1 can be represented in 21 chars. */
-        if(str)
-            sprintf(str, "%d", item->valueint);
+        if(str) sprintf(str, "%d", item->valueint);
     } else {
         str = (char*)cJSON_malloc(64); /* This is a nice tradeoff. */
         if(str) {
@@ -208,12 +193,10 @@ static const char* parse_string(cJSON* item, const char* str)
     } /* not a string! */
 
     while(*ptr != '\"' && *ptr && ++len)
-        if(*ptr++ == '\\')
-            ptr++; /* Skip escaped quotes. */
+        if(*ptr++ == '\\') ptr++; /* Skip escaped quotes. */
 
     out = (char*)cJSON_malloc(len + 1); /* This is how long we need for the string, roughly. */
-    if(!out)
-        return 0;
+    if(!out) return 0;
 
     ptr = str + 1;
     ptr2 = out;
@@ -242,17 +225,14 @@ static const char* parse_string(cJSON* item, const char* str)
                 sscanf(ptr + 1, "%4x", &uc);
                 ptr += 4; /* get the unicode char. */
 
-                if((uc >= 0xDC00 && uc <= 0xDFFF) || uc == 0)
-                    break; // check for invalid.
+                if((uc >= 0xDC00 && uc <= 0xDFFF) || uc == 0) break; // check for invalid.
 
                 if(uc >= 0xD800 && uc <= 0xDBFF) // UTF16 surrogate pairs.
                 {
-                    if(ptr[1] != '\\' || ptr[2] != 'u')
-                        break; // missing second-half of surrogate.
+                    if(ptr[1] != '\\' || ptr[2] != 'u') break; // missing second-half of surrogate.
                     sscanf(ptr + 3, "%4x", &uc2);
                     ptr += 6;
-                    if(uc2 < 0xDC00 || uc2 > 0xDFFF)
-                        break; // invalid second-half of surrogate.
+                    if(uc2 < 0xDC00 || uc2 > 0xDFFF) break; // invalid second-half of surrogate.
                     uc = 0x10000 | ((uc & 0x3FF) << 10) | (uc2 & 0x3FF);
                 }
 
@@ -288,8 +268,7 @@ static const char* parse_string(cJSON* item, const char* str)
         }
     }
     *ptr2 = 0;
-    if(*ptr == '\"')
-        ptr++;
+    if(*ptr == '\"') ptr++;
     item->valuestring = out;
     item->type = cJSON_String;
     return ptr;
@@ -299,12 +278,11 @@ static const char* parse_string(cJSON* item, const char* str)
 static char* print_string_ptr(const char* str)
 {
     const char* ptr;
-    char* ptr2, *out;
+    char *ptr2, *out;
     int len = 0;
     unsigned char token;
 
-    if(!str)
-        return cJSON_strdup("");
+    if(!str) return cJSON_strdup("");
     ptr = str;
     while((token = *ptr) && ++len) {
         if(strchr("\"\\\b\f\n\r\t", token))
@@ -315,8 +293,7 @@ static char* print_string_ptr(const char* str)
     }
 
     out = (char*)cJSON_malloc(len + 3);
-    if(!out)
-        return 0;
+    if(!out) return 0;
 
     ptr2 = out;
     ptr = str;
@@ -360,10 +337,7 @@ static char* print_string_ptr(const char* str)
     return out;
 }
 /* Invote print_string_ptr (which is useful) on an item. */
-static char* print_string(cJSON* item)
-{
-    return print_string_ptr(item->valuestring);
-}
+static char* print_string(cJSON* item) { return print_string_ptr(item->valuestring); }
 
 /* Predeclare these prototypes. */
 static const char* parse_value(cJSON* item, const char* value);
@@ -386,8 +360,7 @@ cJSON* cJSON_Parse(const char* value)
 {
     cJSON* c = cJSON_New_Item();
     ep = 0;
-    if(!c)
-        return 0; /* memory fail */
+    if(!c) return 0; /* memory fail */
 
     if(!parse_value(c, skip(value))) {
         cJSON_Delete(c);
@@ -397,20 +370,13 @@ cJSON* cJSON_Parse(const char* value)
 }
 
 /* Render a cJSON item/entity/structure to text. */
-char* cJSON_Print(cJSON* item)
-{
-    return print_value(item, 0, 1);
-}
-char* cJSON_PrintUnformatted(cJSON* item)
-{
-    return print_value(item, 0, 0);
-}
+char* cJSON_Print(cJSON* item) { return print_value(item, 0, 1); }
+char* cJSON_PrintUnformatted(cJSON* item) { return print_value(item, 0, 0); }
 
 /* Parser core - when encountering text, process appropriately. */
 static const char* parse_value(cJSON* item, const char* value)
 {
-    if(!value)
-        return 0; /* Fail on null. */
+    if(!value) return 0; /* Fail on null. */
     if(!strncmp(value, "null", 4)) {
         item->type = cJSON_NULL;
         return value + 4;
@@ -424,18 +390,10 @@ static const char* parse_value(cJSON* item, const char* value)
         item->valueint = 1;
         return value + 4;
     }
-    if(*value == '\"') {
-        return parse_string(item, value);
-    }
-    if(*value == '-' || (*value >= '0' && *value <= '9')) {
-        return parse_number(item, value);
-    }
-    if(*value == '[') {
-        return parse_array(item, value);
-    }
-    if(*value == '{') {
-        return parse_object(item, value);
-    }
+    if(*value == '\"') { return parse_string(item, value); }
+    if(*value == '-' || (*value >= '0' && *value <= '9')) { return parse_number(item, value); }
+    if(*value == '[') { return parse_array(item, value); }
+    if(*value == '{') { return parse_object(item, value); }
 
     ep = value;
     return 0; /* failure. */
@@ -445,8 +403,7 @@ static const char* parse_value(cJSON* item, const char* value)
 static char* print_value(cJSON* item, int depth, int fmt)
 {
     char* out = 0;
-    if(!item)
-        return 0;
+    if(!item) return 0;
     switch((item->type) & 255) {
     case cJSON_NULL:
         out = cJSON_strdup("null");
@@ -484,30 +441,24 @@ static const char* parse_array(cJSON* item, const char* value)
 
     item->type = cJSON_Array;
     value = skip(value + 1);
-    if(*value == ']')
-        return value + 1; /* empty array. */
+    if(*value == ']') return value + 1; /* empty array. */
 
     item->child = child = cJSON_New_Item();
-    if(!item->child)
-        return 0;                                  /* memory fail */
+    if(!item->child) return 0;                     /* memory fail */
     value = skip(parse_value(child, skip(value))); /* skip any spacing, get the value. */
-    if(!value)
-        return 0;
+    if(!value) return 0;
 
     while(*value == ',') {
         cJSON* new_item;
-        if(!(new_item = cJSON_New_Item()))
-            return 0; /* memory fail */
+        if(!(new_item = cJSON_New_Item())) return 0; /* memory fail */
         child->next = new_item;
         new_item->prev = child;
         child = new_item;
         value = skip(parse_value(child, skip(value + 1)));
-        if(!value)
-            return 0; /* memory fail */
+        if(!value) return 0; /* memory fail */
     }
 
-    if(*value == ']')
-        return value + 1; /* end of array */
+    if(*value == ']') return value + 1; /* end of array */
     ep = value;
     return 0; /* malformed. */
 }
@@ -516,7 +467,7 @@ static const char* parse_array(cJSON* item, const char* value)
 static char* print_array(cJSON* item, int depth, int fmt)
 {
     char** entries;
-    char* out = 0, *ptr, *ret;
+    char *out = 0, *ptr, *ret;
     int len = 5;
     cJSON* child = item->child;
     int numentries = 0, i = 0, fail = 0;
@@ -526,8 +477,7 @@ static char* print_array(cJSON* item, int depth, int fmt)
         numentries++, child = child->next;
     /* Allocate an array to hold the values for each */
     entries = (char**)cJSON_malloc(numentries * sizeof(char*));
-    if(!entries)
-        return 0;
+    if(!entries) return 0;
     memset(entries, 0, numentries * sizeof(char*));
     /* Retrieve all the results: */
     child = item->child;
@@ -542,17 +492,14 @@ static char* print_array(cJSON* item, int depth, int fmt)
     }
 
     /* If we didn't fail, try to malloc the output string */
-    if(!fail)
-        out = (char*)cJSON_malloc(len);
+    if(!fail) out = (char*)cJSON_malloc(len);
     /* If that fails, we fail. */
-    if(!out)
-        fail = 1;
+    if(!out) fail = 1;
 
     /* Handle failure. */
     if(fail) {
         for(i = 0; i < numentries; i++)
-            if(entries[i])
-                cJSON_free(entries[i]);
+            if(entries[i]) cJSON_free(entries[i]);
         cJSON_free(entries);
         return 0;
     }
@@ -566,8 +513,7 @@ static char* print_array(cJSON* item, int depth, int fmt)
         ptr += strlen(entries[i]);
         if(i != numentries - 1) {
             *ptr++ = ',';
-            if(fmt)
-                *ptr++ = ' ';
+            if(fmt) *ptr++ = ' ';
             *ptr = 0;
         }
         cJSON_free(entries[i]);
@@ -589,15 +535,12 @@ static const char* parse_object(cJSON* item, const char* value)
 
     item->type = cJSON_Object;
     value = skip(value + 1);
-    if(*value == '}')
-        return value + 1; /* empty array. */
+    if(*value == '}') return value + 1; /* empty array. */
 
     item->child = child = cJSON_New_Item();
-    if(!item->child)
-        return 0;
+    if(!item->child) return 0;
     value = skip(parse_string(child, skip(value)));
-    if(!value)
-        return 0;
+    if(!value) return 0;
     child->string = child->valuestring;
     child->valuestring = 0;
     if(*value != ':') {
@@ -605,19 +548,16 @@ static const char* parse_object(cJSON* item, const char* value)
         return 0;
     }                                                  /* fail! */
     value = skip(parse_value(child, skip(value + 1))); /* skip any spacing, get the value. */
-    if(!value)
-        return 0;
+    if(!value) return 0;
 
     while(*value == ',') {
         cJSON* new_item;
-        if(!(new_item = cJSON_New_Item()))
-            return 0; /* memory fail */
+        if(!(new_item = cJSON_New_Item())) return 0; /* memory fail */
         child->next = new_item;
         new_item->prev = child;
         child = new_item;
         value = skip(parse_string(child, skip(value + 1)));
-        if(!value)
-            return 0;
+        if(!value) return 0;
         child->string = child->valuestring;
         child->valuestring = 0;
         if(*value != ':') {
@@ -625,12 +565,10 @@ static const char* parse_object(cJSON* item, const char* value)
             return 0;
         }                                                  /* fail! */
         value = skip(parse_value(child, skip(value + 1))); /* skip any spacing, get the value. */
-        if(!value)
-            return 0;
+        if(!value) return 0;
     }
 
-    if(*value == '}')
-        return value + 1; /* end of array */
+    if(*value == '}') return value + 1; /* end of array */
     ep = value;
     return 0; /* malformed. */
 }
@@ -638,8 +576,8 @@ static const char* parse_object(cJSON* item, const char* value)
 /* Render an object to text. */
 static char* print_object(cJSON* item, int depth, int fmt)
 {
-    char** entries = 0, **names = 0;
-    char* out = 0, *ptr, *ret, *str;
+    char **entries = 0, **names = 0;
+    char *out = 0, *ptr, *ret, *str;
     int len = 7, i = 0, j;
     cJSON* child = item->child;
     int numentries = 0, fail = 0;
@@ -648,8 +586,7 @@ static char* print_object(cJSON* item, int depth, int fmt)
         numentries++, child = child->next;
     /* Allocate space for the names and the objects */
     entries = (char**)cJSON_malloc(numentries * sizeof(char*));
-    if(!entries)
-        return 0;
+    if(!entries) return 0;
     names = (char**)cJSON_malloc(numentries * sizeof(char*));
     if(!names) {
         cJSON_free(entries);
@@ -661,8 +598,7 @@ static char* print_object(cJSON* item, int depth, int fmt)
     /* Collect all the results into our arrays: */
     child = item->child;
     depth++;
-    if(fmt)
-        len += depth;
+    if(fmt) len += depth;
     while(child) {
         names[i] = str = print_string_ptr(child->string);
         entries[i++] = ret = print_value(child, depth, fmt);
@@ -674,18 +610,14 @@ static char* print_object(cJSON* item, int depth, int fmt)
     }
 
     /* Try to allocate the output string */
-    if(!fail)
-        out = (char*)cJSON_malloc(len);
-    if(!out)
-        fail = 1;
+    if(!fail) out = (char*)cJSON_malloc(len);
+    if(!out) fail = 1;
 
     /* Handle failure */
     if(fail) {
         for(i = 0; i < numentries; i++) {
-            if(names[i])
-                cJSON_free(names[i]);
-            if(entries[i])
-                cJSON_free(entries[i]);
+            if(names[i]) cJSON_free(names[i]);
+            if(entries[i]) cJSON_free(entries[i]);
         }
         cJSON_free(names);
         cJSON_free(entries);
@@ -695,8 +627,7 @@ static char* print_object(cJSON* item, int depth, int fmt)
     /* Compose the output: */
     *out = '{';
     ptr = out + 1;
-    if(fmt)
-        *ptr++ = '\n';
+    if(fmt) *ptr++ = '\n';
     *ptr = 0;
     for(i = 0; i < numentries; i++) {
         if(fmt)
@@ -705,14 +636,11 @@ static char* print_object(cJSON* item, int depth, int fmt)
         strcpy(ptr, names[i]);
         ptr += strlen(names[i]);
         *ptr++ = ':';
-        if(fmt)
-            *ptr++ = FMT_WHITESPACE_CHAR;
+        if(fmt) *ptr++ = FMT_WHITESPACE_CHAR;
         strcpy(ptr, entries[i]);
         ptr += strlen(entries[i]);
-        if(i != numentries - 1)
-            *ptr++ = ',';
-        if(fmt)
-            *ptr++ = '\n';
+        if(i != numentries - 1) *ptr++ = ',';
+        if(fmt) *ptr++ = '\n';
         *ptr = 0;
         cJSON_free(names[i]);
         cJSON_free(entries[i]);
@@ -762,8 +690,7 @@ static void suffix_object(cJSON* prev, cJSON* item)
 static cJSON* create_reference(cJSON* item)
 {
     cJSON* ref = cJSON_New_Item();
-    if(!ref)
-        return 0;
+    if(!ref) return 0;
     memcpy(ref, item, sizeof(cJSON));
     ref->string = 0;
     ref->type |= cJSON_IsReference;
@@ -775,8 +702,7 @@ static cJSON* create_reference(cJSON* item)
 void cJSON_AddItemToArray(cJSON* array, cJSON* item)
 {
     cJSON* c = array->child;
-    if(!item)
-        return;
+    if(!item) return;
     if(!c) {
         array->child = item;
     } else {
@@ -787,17 +713,12 @@ void cJSON_AddItemToArray(cJSON* array, cJSON* item)
 }
 void cJSON_AddItemToObject(cJSON* object, const char* string, cJSON* item)
 {
-    if(!item)
-        return;
-    if(item->string)
-        cJSON_free(item->string);
+    if(!item) return;
+    if(item->string) cJSON_free(item->string);
     item->string = cJSON_strdup(string);
     cJSON_AddItemToArray(object, item);
 }
-void cJSON_AddItemReferenceToArray(cJSON* array, cJSON* item)
-{
-    cJSON_AddItemToArray(array, create_reference(item));
-}
+void cJSON_AddItemReferenceToArray(cJSON* array, cJSON* item) { cJSON_AddItemToArray(array, create_reference(item)); }
 void cJSON_AddItemReferenceToObject(cJSON* object, const char* string, cJSON* item)
 {
     cJSON_AddItemToObject(object, string, create_reference(item));
@@ -808,29 +729,21 @@ cJSON* cJSON_DetachItemFromArray(cJSON* array, int which)
     cJSON* c = array->child;
     while(c && which > 0)
         c = c->next, which--;
-    if(!c)
-        return 0;
-    if(c->prev)
-        c->prev->next = c->next;
-    if(c->next)
-        c->next->prev = c->prev;
-    if(c == array->child)
-        array->child = c->next;
+    if(!c) return 0;
+    if(c->prev) c->prev->next = c->next;
+    if(c->next) c->next->prev = c->prev;
+    if(c == array->child) array->child = c->next;
     c->prev = c->next = 0;
     return c;
 }
-void cJSON_DeleteItemFromArray(cJSON* array, int which)
-{
-    cJSON_Delete(cJSON_DetachItemFromArray(array, which));
-}
+void cJSON_DeleteItemFromArray(cJSON* array, int which) { cJSON_Delete(cJSON_DetachItemFromArray(array, which)); }
 cJSON* cJSON_DetachItemFromObject(cJSON* object, const char* string)
 {
     int i = 0;
     cJSON* c = object->child;
     while(c && cJSON_strcasecmp(c->string, string))
         i++, c = c->next;
-    if(c)
-        return cJSON_DetachItemFromArray(object, i);
+    if(c) return cJSON_DetachItemFromArray(object, i);
     return 0;
 }
 void cJSON_DeleteItemFromObject(cJSON* object, const char* string)
@@ -844,12 +757,10 @@ void cJSON_ReplaceItemInArray(cJSON* array, int which, cJSON* newitem)
     cJSON* c = array->child;
     while(c && which > 0)
         c = c->next, which--;
-    if(!c)
-        return;
+    if(!c) return;
     newitem->next = c->next;
     newitem->prev = c->prev;
-    if(newitem->next)
-        newitem->next->prev = newitem;
+    if(newitem->next) newitem->next->prev = newitem;
     if(c == array->child)
         array->child = newitem;
     else
@@ -873,29 +784,25 @@ void cJSON_ReplaceItemInObject(cJSON* object, const char* string, cJSON* newitem
 cJSON* cJSON_CreateNull()
 {
     cJSON* item = cJSON_New_Item();
-    if(item)
-        item->type = cJSON_NULL;
+    if(item) item->type = cJSON_NULL;
     return item;
 }
 cJSON* cJSON_CreateTrue()
 {
     cJSON* item = cJSON_New_Item();
-    if(item)
-        item->type = cJSON_True;
+    if(item) item->type = cJSON_True;
     return item;
 }
 cJSON* cJSON_CreateFalse()
 {
     cJSON* item = cJSON_New_Item();
-    if(item)
-        item->type = cJSON_False;
+    if(item) item->type = cJSON_False;
     return item;
 }
 cJSON* cJSON_CreateBool(int b)
 {
     cJSON* item = cJSON_New_Item();
-    if(item)
-        item->type = b ? cJSON_True : cJSON_False;
+    if(item) item->type = b ? cJSON_True : cJSON_False;
     return item;
 }
 cJSON* cJSON_CreateNumber(double num)
@@ -920,15 +827,13 @@ cJSON* cJSON_CreateString(const char* string)
 cJSON* cJSON_CreateArray()
 {
     cJSON* item = cJSON_New_Item();
-    if(item)
-        item->type = cJSON_Array;
+    if(item) item->type = cJSON_Array;
     return item;
 }
 cJSON* cJSON_CreateObject()
 {
     cJSON* item = cJSON_New_Item();
-    if(item)
-        item->type = cJSON_Object;
+    if(item) item->type = cJSON_Object;
     return item;
 }
 
@@ -936,7 +841,7 @@ cJSON* cJSON_CreateObject()
 cJSON* cJSON_CreateIntArray(int* numbers, int count)
 {
     int i;
-    cJSON* n = 0, *p = 0, *a = cJSON_CreateArray();
+    cJSON *n = 0, *p = 0, *a = cJSON_CreateArray();
     for(i = 0; a && i < count; i++) {
         n = cJSON_CreateNumber(numbers[i]);
         if(!i)
@@ -950,7 +855,7 @@ cJSON* cJSON_CreateIntArray(int* numbers, int count)
 cJSON* cJSON_CreateFloatArray(float* numbers, int count)
 {
     int i;
-    cJSON* n = 0, *p = 0, *a = cJSON_CreateArray();
+    cJSON *n = 0, *p = 0, *a = cJSON_CreateArray();
     for(i = 0; a && i < count; i++) {
         n = cJSON_CreateNumber(numbers[i]);
         if(!i)
@@ -964,7 +869,7 @@ cJSON* cJSON_CreateFloatArray(float* numbers, int count)
 cJSON* cJSON_CreateDoubleArray(double* numbers, int count)
 {
     int i;
-    cJSON* n = 0, *p = 0, *a = cJSON_CreateArray();
+    cJSON *n = 0, *p = 0, *a = cJSON_CreateArray();
     for(i = 0; a && i < count; i++) {
         n = cJSON_CreateNumber(numbers[i]);
         if(!i)
@@ -978,7 +883,7 @@ cJSON* cJSON_CreateDoubleArray(double* numbers, int count)
 cJSON* cJSON_CreateStringArray(const char** strings, int count)
 {
     int i;
-    cJSON* n = 0, *p = 0, *a = cJSON_CreateArray();
+    cJSON *n = 0, *p = 0, *a = cJSON_CreateArray();
     for(i = 0; a && i < count; i++) {
         n = cJSON_CreateString(strings[i]);
         if(!i)
