@@ -35,12 +35,105 @@
 #include <wx/arrstr.h>
 #include <wx/event.h>
 #include <wx/sharedptr.h>
+#include "LSP/CompletionItem.h"
 
 // Set of flags that can be passed within the 'S{G}etInt' function of clCommandEvent
 enum {
     kEventImportingFolder = 0x00000001,
 };
 
+#if !wxUSE_GUI
+// To allow building libcodelite without UI we need these 2 event types (wxCommandEvent and wxNotifyEvent)
+class WXDLLIMPEXP_CL wxCommandEvent : public wxEvent, public wxEventBasicPayloadMixin
+{
+public:
+    wxCommandEvent(wxEventType commandType = wxEVT_NULL, int winid = 0)
+        : wxEvent(winid, commandType)
+    {
+        m_clientData = NULL;
+        m_clientObject = NULL;
+        m_isCommandEvent = true;
+
+        // the command events are propagated upwards by default
+        m_propagationLevel = wxEVENT_PROPAGATE_MAX;
+    }
+
+    wxCommandEvent(const wxCommandEvent& event)
+        : wxEvent(event)
+        , wxEventBasicPayloadMixin(event)
+        , m_clientData(event.m_clientData)
+        , m_clientObject(event.m_clientObject)
+    {
+        // Because GetString() can retrieve the string text only on demand, we
+        // need to copy it explicitly.
+        if(m_cmdString.empty()) m_cmdString = event.GetString();
+    }
+
+    virtual ~wxCommandEvent() {}
+
+    // Set/Get client data from controls
+    void SetClientData(void* clientData) { m_clientData = clientData; }
+    void* GetClientData() const { return m_clientData; }
+
+    // Set/Get client object from controls
+    void SetClientObject(wxClientData* clientObject) { m_clientObject = clientObject; }
+    wxClientData* GetClientObject() const { return m_clientObject; }
+
+    // Note: this shadows wxEventBasicPayloadMixin::GetString(), because it does some
+    // GUI-specific hacks
+    wxString GetString() const { return m_cmdString; }
+
+    // Get listbox selection if single-choice
+    int GetSelection() const { return m_commandInt; }
+
+    // Get checkbox value
+    bool IsChecked() const { return m_commandInt != 0; }
+
+    // true if the listbox event was a selection.
+    bool IsSelection() const { return (m_extraLong != 0); }
+
+    virtual wxEvent* Clone() const wxOVERRIDE { return new wxCommandEvent(*this); }
+    virtual wxEventCategory GetEventCategory() const wxOVERRIDE { return wxEVT_CATEGORY_USER_INPUT; }
+
+protected:
+    void* m_clientData;           // Arbitrary client data
+    wxClientData* m_clientObject; // Arbitrary client object
+};
+
+// this class adds a possibility to react (from the user) code to a control
+// notification: allow or veto the operation being reported.
+class WXDLLIMPEXP_CL wxNotifyEvent : public wxCommandEvent
+{
+public:
+    wxNotifyEvent(wxEventType commandType = wxEVT_NULL, int winid = 0)
+        : wxCommandEvent(commandType, winid)
+    {
+        m_bAllow = true;
+    }
+
+    wxNotifyEvent(const wxNotifyEvent& event)
+        : wxCommandEvent(event)
+    {
+        m_bAllow = event.m_bAllow;
+    }
+
+    // veto the operation (usually it's allowed by default)
+    void Veto() { m_bAllow = false; }
+
+    // allow the operation if it was disabled by default
+    void Allow() { m_bAllow = true; }
+
+    // for implementation code only: is the operation allowed?
+    bool IsAllowed() const { return m_bAllow; }
+
+    virtual wxEvent* Clone() const wxOVERRIDE { return new wxNotifyEvent(*this); }
+
+private:
+    bool m_bAllow;
+};
+
+#endif // !wxUSE_GUI
+
 /// a wxCommandEvent that takes ownership of the clientData
 class WXDLLIMPEXP_CL clCommandEvent : public wxCommandEvent
 {
@@ -136,6 +229,7 @@ class WXDLLIMPEXP_CL clCodeCompletionEvent : public clCommandEvent
     wxCodeCompletionBoxEntry::Ptr_t m_entry;
     wxArrayString m_definitions;
     wxCodeCompletionBoxEntry::Vec_t m_entries;
+    LSP::CompletionItem::eTriggerKind m_triggerKind = LSP::CompletionItem::kTriggerUnknown;
 
 public:
     clCodeCompletionEvent(wxEventType commandType = wxEVT_NULL, int winid = 0);
@@ -157,6 +251,9 @@ public:
     const wxCodeCompletionBoxEntry::Vec_t& GetEntries() const { return m_entries; }
     wxCodeCompletionBoxEntry::Vec_t& GetEntries() { return m_entries; }
 
+    const LSP::CompletionItem::eTriggerKind& GetTriggerKind() const { return m_triggerKind; }
+    void SetTriggerKind(const LSP::CompletionItem::eTriggerKind& triggerKind) { this->m_triggerKind = triggerKind; }
+    
     /**
      * @brief return the Editor object
      */
@@ -181,10 +278,12 @@ typedef void (wxEvtHandler::*clCodeCompletionEventFunction)(clCodeCompletionEven
 
 class WXDLLIMPEXP_CL clColourEvent : public clCommandEvent
 {
+#if wxUSE_GUI
     wxColour m_bgColour;
     wxColour m_fgColour;
     wxColour m_borderColour;
     wxWindow* m_page;
+#endif
     bool m_isActiveTab;
 
 public:
@@ -194,6 +293,7 @@ public:
     virtual ~clColourEvent();
     virtual wxEvent* Clone() const { return new clColourEvent(*this); };
 
+#if wxUSE_GUI
     void SetBorderColour(const wxColour& borderColour) { this->m_borderColour = borderColour; }
     const wxColour& GetBorderColour() const { return m_borderColour; }
     void SetPage(wxWindow* page) { this->m_page = page; }
@@ -202,6 +302,7 @@ public:
     void SetFgColour(const wxColour& fgColour) { this->m_fgColour = fgColour; }
     const wxColour& GetBgColour() const { return m_bgColour; }
     const wxColour& GetFgColour() const { return m_fgColour; }
+#endif
     void SetIsActiveTab(bool isActiveTab) { this->m_isActiveTab = isActiveTab; }
     bool IsActiveTab() const { return m_isActiveTab; }
 };
@@ -219,6 +320,8 @@ class WXDLLIMPEXP_CL clBuildEvent : public clCommandEvent
     bool m_projectOnly;
     size_t m_warningCount;
     size_t m_errorCount;
+    wxString m_kind;
+    bool m_isRunning;
 
 public:
     clBuildEvent(wxEventType commandType = wxEVT_NULL, int winid = 0);
@@ -227,6 +330,12 @@ public:
     virtual ~clBuildEvent();
     virtual wxEvent* Clone() const { return new clBuildEvent(*this); };
 
+    void SetIsRunning(bool isRunning) { this->m_isRunning = isRunning; }
+    bool IsRunning() const { return m_isRunning; }
+
+    void SetKind(const wxString& kind) { this->m_kind = kind; }
+    const wxString& GetKind() const { return this->m_kind; }
+
     void SetProjectOnly(bool projectOnly) { this->m_projectOnly = projectOnly; }
     bool IsProjectOnly() const { return m_projectOnly; }
     void SetCommand(const wxString& command) { this->m_command = command; }
@@ -260,6 +369,9 @@ class WXDLLIMPEXP_CL clDebugEvent : public clCommandEvent
     wxString m_arguments;        // wxEVT_DBG_UI_QUICK_DEBUG
     wxString m_startupCommands;  // wxEVT_DBG_UI_QUICK_DEBUG
     size_t m_features;
+    wxString m_memoryAddress;    // wxEVT_DEBUGGER_SET_MEMORY
+    wxString m_memoryBlockValue; // wxEVT_DEBUGGER_SET_MEMORY
+    size_t m_memoryBlockSize;    // wxEVT_DEBUGGER_SET_MEMORY
 
 public:
     // Special features not available by all the debuggers
@@ -299,6 +411,12 @@ public:
     void SetProjectName(const wxString& projectName) { this->m_projectName = projectName; }
     const wxString& GetConfigurationName() const { return m_configurationName; }
     const wxString& GetProjectName() const { return m_projectName; }
+    void SetMemoryAddress(const wxString& memoryAddress) { this->m_memoryAddress = memoryAddress; }
+    void SetMemoryBlockSize(size_t memoryBlockSize) { this->m_memoryBlockSize = memoryBlockSize; }
+    void SetMemoryBlockValue(const wxString& memoryBlockValue) { this->m_memoryBlockValue = memoryBlockValue; }
+    const wxString& GetMemoryAddress() const { return m_memoryAddress; }
+    size_t GetMemoryBlockSize() const { return m_memoryBlockSize; }
+    const wxString& GetMemoryBlockValue() const { return m_memoryBlockValue; }
 };
 
 typedef void (wxEvtHandler::*clDebugEventFunction)(clDebugEvent&);
@@ -452,7 +570,9 @@ typedef void (wxEvtHandler::*clSourceFormatEventFunction)(clSourceFormatEvent&);
 //---------------------------------------------------------------
 class WXDLLIMPEXP_CL clContextMenuEvent : public clCommandEvent
 {
+#if wxUSE_GUI
     wxMenu* m_menu;
+#endif
     wxObject* m_editor;
     wxString m_path;
 
@@ -462,8 +582,10 @@ public:
     clContextMenuEvent& operator=(const clContextMenuEvent& src);
     virtual ~clContextMenuEvent();
     virtual wxEvent* Clone() const { return new clContextMenuEvent(*this); }
+#if wxUSE_GUI
     void SetMenu(wxMenu* menu) { this->m_menu = menu; }
     wxMenu* GetMenu() { return m_menu; }
+#endif
     void SetEditor(wxObject* editor) { this->m_editor = editor; }
     wxObject* GetEditor() { return m_editor; }
     const wxString& GetPath() const { return m_path; }
@@ -538,6 +660,35 @@ public:
 typedef void (wxEvtHandler::*clFindEventFunction)(clFindEvent&);
 #define clFindEventHandler(func) wxEVENT_HANDLER_CAST(clFindEventFunction, func)
 
+//---------------------------------------------------------------
+// Find in files event
+//---------------------------------------------------------------
+class WXDLLIMPEXP_CL clFindInFilesEvent : public clCommandEvent
+{
+    wxString m_paths;
+    wxString m_fileMask;
+    size_t m_options = 0;
+    wxString m_transientPaths;
+
+public:
+    clFindInFilesEvent(wxEventType commandType = wxEVT_NULL, int winid = 0);
+    clFindInFilesEvent(const clFindInFilesEvent& event);
+    clFindInFilesEvent& operator=(const clFindInFilesEvent& src);
+    virtual ~clFindInFilesEvent();
+    virtual wxEvent* Clone() const { return new clFindInFilesEvent(*this); }
+    void SetFileMask(const wxString& fileMask) { this->m_fileMask = fileMask; }
+    void SetOptions(size_t options) { this->m_options = options; }
+    const wxString& GetFileMask() const { return m_fileMask; }
+    size_t GetOptions() const { return m_options; }
+    void SetPaths(const wxString& paths) { this->m_paths = paths; }
+    const wxString& GetPaths() const { return m_paths; }
+    void SetTransientPaths(const wxString& transientPaths) { this->m_transientPaths = transientPaths; }
+    const wxString& GetTransientPaths() const { return m_transientPaths; }
+};
+
+typedef void (wxEvtHandler::*clFindInFilesEventFunction)(clFindInFilesEvent&);
+#define clFindInFilesEventHandler(func) wxEVENT_HANDLER_CAST(clFindInFilesEventFunction, func)
+
 // --------------------------------------------------------------
 // Parsing event
 // --------------------------------------------------------------
