@@ -42,7 +42,7 @@
 #include <wx/ffile.h>
 #include <wx/stopwatch.h>
 #include <wx/tokenzr.h>
-#include <wx/xrc/xmlres.h>
+#include "fileextmanager.h"
 
 #define DEBUG_MESSAGE(x) CL_DEBUG1(x.c_str())
 
@@ -105,6 +105,22 @@ void ParseThread::ProcessRequest(ThreadRequest* request)
 {
     // request is delete by the parent WorkerThread after this method is completed
     ParseRequest* req = (ParseRequest*)request;
+    FileLogger::RegisterThread(wxThread::GetCurrentId(), "C++ Parser Thread");
+
+    // Filter non C++ files
+    if(!req->_workspaceFiles.empty()) {
+        std::vector<std::string> filtered_list;
+        filtered_list.reserve(req->_workspaceFiles.size());
+        for(std::string& filename : req->_workspaceFiles) {
+            if(FileExtManager::IsCxxFile(wxString() << filename)) { filtered_list.push_back(std::move(filename)); }
+        }
+        req->_workspaceFiles.swap(filtered_list);
+    }
+
+    wxArrayString inc, exc;
+    GetSearchPaths(inc, exc);
+
+    clDEBUG1() << "include paths:\n" << inc;
 
     switch(req->getType()) {
     case ParseRequest::PR_PARSEINCLUDES:
@@ -437,9 +453,6 @@ void ParseThread::ProcessParseAndStore(ParseRequest* req)
             wxCommandEvent retaggingProgressEvent(wxEVT_PARSE_THREAD_RETAGGING_PROGRESS);
             retaggingProgressEvent.SetInt((int)precent);
             req->_evtHandler->AddPendingEvent(retaggingProgressEvent);
-
-        } else if(lastPercentageReported != precent) {
-            wxPrintf(wxT("parsing: %%%d completed\n"), precent);
         }
 
         TagTreePtr tree = TagsManagerST::Get()->ParseSourceFile(curFile);
@@ -640,12 +653,9 @@ void ParseThread::ProcessColourRequest(ParseRequest* req)
 {
     CxxTokenizer tokenizer;
     // read the file content
-    wxFFile fp(req->getFile(), "rb");
-    if(fp.IsOpened()) {
+    wxString content;
+    if(FileUtils::ReadFileContent(req->getFile(), content)) {
         wxString flatStrLocals, flatClasses;
-        wxString content;
-        fp.ReadAll(&content);
-        fp.Close();
 
         tokenizer.Reset(content);
 
@@ -669,17 +679,13 @@ void ParseThread::ProcessColourRequest(ParseRequest* req)
         db->OpenDatabase(req->getDbfile());
 
         std::vector<wxString> nonWorkspaceSymbols, workspaceSymbols;
-        clDEBUG1() << "Parse Thread: removing non workspace symbols" << clEndl;
         db->RemoveNonWorkspaceSymbols(tokensArr, workspaceSymbols, nonWorkspaceSymbols);
-        clDEBUG1() << "Parse Thread: removing non workspace symbols...done" << clEndl;
 
         // Convert the output to a space delimited array
         std::for_each(workspaceSymbols.begin(), workspaceSymbols.end(),
                       [&](const wxString& token) { flatClasses << token << " "; });
         std::for_each(nonWorkspaceSymbols.begin(), nonWorkspaceSymbols.end(),
                       [&](const wxString& token) { flatStrLocals << token << " "; });
-        clDEBUG1() << "The following local variables were found:\n" << flatStrLocals << clEndl;
-        clDEBUG1() << "The following classes were found:\n" << flatClasses << clEndl;
 
         if(req->_evtHandler) {
             clCommandEvent event(wxEVT_PARSE_THREAD_SUGGEST_COLOUR_TOKENS);
@@ -708,3 +714,64 @@ void ParseThread::ProcessSourceToTags(ParseRequest* req)
     event.SetInt(req->_uid); // send back the unique ID
     req->_evtHandler->AddPendingEvent(event);
 }
+
+void ParseThread::AddPaths(const wxArrayString& inc, const wxArrayString& exc)
+{
+    // remove all duplicates
+    wxCriticalSectionLocker locker(m_cs);
+    {
+        wxStringSet_t unique;
+        wxArrayString tmp;
+        for(const wxString& path : inc) {
+            wxFileName fnPath(path, "");
+            wxString canonPath = fnPath.GetPath();
+            if(FileUtils::IsDirectory(canonPath)) {
+                if(unique.count(canonPath) == 0) {
+                    unique.insert(canonPath);
+                    tmp.Add(canonPath);
+                }
+            }
+        }
+        for(const wxString& path : m_searchPaths) {
+            wxFileName fnPath(path, "");
+            wxString canonPath = fnPath.GetPath();
+            if(FileUtils::IsDirectory(canonPath)) {
+                if(unique.count(canonPath) == 0) {
+                    unique.insert(canonPath);
+                    tmp.Add(canonPath);
+                }
+            }
+        }
+        m_searchPaths.swap(tmp);
+        std::sort(m_searchPaths.begin(), m_searchPaths.end());
+    }
+    
+    {
+        wxStringSet_t unique;
+        wxArrayString tmp;
+        for(const wxString& path : exc) {
+            wxFileName fnPath(path, "");
+            wxString canonPath = fnPath.GetPath();
+            if(FileUtils::IsDirectory(canonPath)) {
+                if(unique.count(canonPath) == 0) {
+                    unique.insert(canonPath);
+                    tmp.Add(canonPath);
+                }
+            }
+        }
+        for(const wxString& path : m_excludePaths) {
+            wxFileName fnPath(path, "");
+            wxString canonPath = fnPath.GetPath();
+            if(FileUtils::IsDirectory(canonPath)) {
+                if(unique.count(canonPath) == 0) {
+                    unique.insert(canonPath);
+                    tmp.Add(canonPath);
+                }
+            }
+        }
+        m_excludePaths.swap(tmp);
+        std::sort(m_excludePaths.begin(), m_excludePaths.end());
+    }
+}
+
+void ParseThread::ClearPaths() {}
