@@ -36,6 +36,7 @@
 #include <list>
 #include <map>
 #include "optionsconfig.h"
+#include "LSP/basic_types.h"
 
 class wxStyledTextCtrl;
 
@@ -95,6 +96,11 @@ public:
      */
     virtual bool IsModified() = 0;
 
+    /**
+     * @brief Get the editor's modification count
+     */
+    virtual wxUint64 GetModificationCount() const = 0;
+
     /**
      * \brief return the current editor content
      */
@@ -135,7 +141,7 @@ public:
      * to the  current file
      */
     virtual void OpenFile() = 0;
-    
+
     /**
      * @brief reload file content from the disk
      * @param keepUndoHistory
@@ -145,12 +151,12 @@ public:
      * @brief save the editor
      */
     virtual bool Save() = 0;
-    
+
     /**
      * @brief save the current editor with a different name
      */
     virtual bool SaveAs(const wxString& defaultName = wxEmptyString, const wxString& savePath = wxEmptyString) = 0;
-    
+
     /**
      * \brief return the current position of the caret
      */
@@ -170,7 +176,7 @@ public:
     /**
      * \brief return the current word under the caret. May return wxEmptyString
      */
-    virtual wxString GetWordAtCaret() = 0;
+    virtual wxString GetWordAtCaret(bool wordCharsOnly = true) = 0;
 
     /**
      * @brief return the word under the mouse pointer.
@@ -178,6 +184,13 @@ public:
      */
     virtual void GetWordAtMousePointer(wxString& word, wxRect& wordRect) = 0;
 
+    /**
+     * @brief get word at a given position
+     * @param pos word's position
+     * @param wordCharsOnly when set to false, return the string between the nearest whitespaces
+     */
+    virtual wxString GetWordAtPosition(int pos, bool wordCharsOnly = true) = 0;
+
     /**
      * @brief return the EOL mode of the editor.
      * 	wxSCI_EOL_CRLF 	0
@@ -328,6 +341,12 @@ public:
      * @return return true if a match was found, false otherwise
      */
     virtual bool FindAndSelect(const wxString& pattern, const wxString& what, int from_pos, NavMgr* navmgr) = 0;
+    
+    /**
+     * @brief select range
+     */
+    virtual bool SelectRange(const LSP::Range& range) = 0;
+    
     /**
      * @brief Similar to the above but returns void, and is implemented asynchronously
      */
@@ -471,9 +490,7 @@ public:
     wxClientData* GetClientData(const wxString& key) const
     {
         IEditor::ClientDataMap_t::const_iterator iter = m_data.find(key);
-        if(iter != m_data.end()) {
-            return iter->second;
-        }
+        if(iter != m_data.end()) { return iter->second; }
         return NULL;
     }
 
@@ -499,17 +516,17 @@ public:
      * @brief return a string representing all the local variables coloured by this editor
      */
     virtual const wxString& GetKeywordLocals() const = 0;
-    
+
     /**
      * @brief get the options associated with this editor
      */
     virtual OptionsConfigPtr GetOptions() = 0;
-    
+
     /**
      * @brief apply editor configuration (TAB vs SPACES, tab size, EOL mode etc)
      */
     virtual void ApplyEditorConfig() = 0;
-    
+
     /**
      * @brief return list of bookmarks for a given editor
      * @param editor the editor
