@@ -69,7 +69,7 @@ protected:
     void SaveSearchData();
     void LoadSearch(const History& h);
     virtual void OnFindInFiles(wxCommandEvent& e);
-    virtual void OnRecentSearches(wxAuiToolBarEvent& e);
+    virtual void OnRecentSearches(wxCommandEvent& e);
     virtual void OnSearchStart(wxCommandEvent& e);
     virtual void OnSearchMatch(wxCommandEvent& e);
     virtual void OnSearchEnded(wxCommandEvent& e);
