@@ -37,6 +37,7 @@
 #include "drawingutils.h"
 #include "event_notifier.h"
 #include <algorithm>
+#include <wx/utils.h>
 
 BEGIN_EVENT_TABLE(LocalsTable, DebuggerTreeListCtrlBase)
 EVT_MENU(XRCID("Change_Value"), LocalsTable::OnEditValue)
@@ -48,17 +49,21 @@ LocalsTable::LocalsTable(wxWindow* parent)
     , m_arrayAsCharPtr(false)
     , m_sortAsc(true)
 {
-    m_listTable->AddColumn(_("Name"), 150);
-    m_listTable->AddColumn(_("Value"), 500);
-    m_listTable->AddColumn(_("Type"), 200);
     m_listTable->AddRoot(_("Locals"));
-
+    m_listTable->AddHeader(_("Name"));
+    m_listTable->AddHeader(_("Value"));
+    m_listTable->AddHeader(_("Type"));
+    
+    // Only sort top level items, don't sort their children
+    m_listTable->AddTreeStyle(wxTR_SORT_TOP_LEVEL);
+    
+    SetSortingFunction();
     m_DBG_USERR = DBG_USERR_LOCALS;
     m_QUERY_NUM_CHILDS = QUERY_LOCALS_CHILDS;
     m_LIST_CHILDS = LIST_LOCALS_CHILDS;
 
-    EventNotifier::Get()->Connect(
-        wxEVT_DEBUGGER_FRAME_SELECTED, clCommandEventHandler(LocalsTable::OnStackSelected), NULL, this);
+    EventNotifier::Get()->Connect(wxEVT_DEBUGGER_FRAME_SELECTED, clCommandEventHandler(LocalsTable::OnStackSelected),
+                                  NULL, this);
 }
 
 LocalsTable::~LocalsTable() {}
@@ -100,7 +105,7 @@ void LocalsTable::OnCreateVariableObj(const DebuggerEventData& event)
             data->_kind = DbgTreeItemData::VariableObject;
 
             // variable object's type name is extracted from the event.m_variableObject.typeName
-            m_listTable->SetItemText(iter->second, 2, event.m_variableObject.typeName);
+            m_listTable->SetItemText(iter->second, event.m_variableObject.typeName, 2);
 
             // refresh this item only
             IDebugger* dbgr = DoGetDebugger();
@@ -144,12 +149,10 @@ void LocalsTable::OnListChildren(const DebuggerEventData& event)
                     data->_isFake = ch.isAFake;
 
                     wxTreeItemId child = m_listTable->AppendItem(item, ch.varName, -1, -1, data);
-                    m_listTable->SetItemText(child, 2, ch.type);
+                    m_listTable->SetItemText(child, ch.type, 2);
 
                     // Add a dummy node
-                    if(child.IsOk() && ch.numChilds > 0) {
-                        m_listTable->AppendItem(child, wxT("<dummy>"));
-                    }
+                    if(child.IsOk() && ch.numChilds > 0) { m_listTable->AppendItem(child, wxT("<dummy>")); }
 
                     // refresh this item only
                     dbgr->EvaluateVariableObject(data->_gdbId, m_DBG_USERR);
@@ -185,6 +188,7 @@ void LocalsTable::OnVariableObjUpdate(const DebuggerEventData& event)
 
 void LocalsTable::OnItemExpanding(wxTreeEvent& event)
 {
+    wxBusyCursor bc;
     wxTreeItemIdValue cookie;
     wxTreeItemId child = m_listTable->GetFirstChild(event.GetItem(), cookie);
 
@@ -225,9 +229,7 @@ void LocalsTable::OnItemExpanding(wxTreeEvent& event)
             // create a variable object
             wxString expres = m_listTable->GetItemText(event.GetItem());
             DbgTreeItemData* data = (DbgTreeItemData*)m_listTable->GetItemData(event.GetItem());
-            if(data && data->_kind == DbgTreeItemData::FuncRetValue) {
-                expres = data->_retValueGdbValue;
-            }
+            if(data && data->_kind == DbgTreeItemData::FuncRetValue) { expres = data->_retValueGdbValue; }
 
             dbgr->CreateVariableObject(expres,                                       // the expression
                                        data->_kind == DbgTreeItemData::FuncRetValue, // bound to creation frame?
@@ -237,8 +239,7 @@ void LocalsTable::OnItemExpanding(wxTreeEvent& event)
     }
 }
 
-void LocalsTable::DoClearNonVariableObjectEntries(wxArrayString& itemsNotRemoved,
-                                                  size_t flags,
+void LocalsTable::DoClearNonVariableObjectEntries(wxArrayString& itemsNotRemoved, size_t flags,
                                                   std::map<wxString, wxString>& oldValues)
 {
     wxTreeItemIdValue cookie;
@@ -269,19 +270,9 @@ void LocalsTable::DoClearNonVariableObjectEntries(wxArrayString& itemsNotRemoved
 void LocalsTable::DoUpdateLocals(const LocalVariables& localsUnSorted, size_t kind)
 {
     LocalVariables locals = localsUnSorted;
-    std::sort(locals.begin(), locals.end(), [&](const LocalVariable& v1, const LocalVariable& v2) {
-        if(m_sortAsc) {
-            return v1.name.Lower().CmpNoCase(v2.name.Lower()) < 0;
-        } else {
-            return v1.name.Lower().CmpNoCase(v2.name.Lower()) >= 0;
-        }
-    });
-
     wxTreeItemId root = m_listTable->GetRootItem();
     if(!root.IsOk()) return;
 
-    wxColour rootItemColour = DrawingUtils::LightColour(wxT("LIGHT GRAY"), 3.0);
-
     IDebugger* dbgr = DoGetDebugger();
     wxArrayString itemsNotRemoved;
     // remove the non-variable objects and return a list
@@ -293,9 +284,7 @@ void LocalsTable::DoUpdateLocals(const LocalVariables& localsUnSorted, size_t ki
 
         // try to replace the
         wxString newVarName;
-        if(m_resolveLocals) {
-            newVarName = m_preDefTypes.GetPreDefinedTypeForTypename(locals[i].type, locals[i].name);
-        }
+        if(m_resolveLocals) { newVarName = m_preDefTypes.GetPreDefinedTypeForTypename(locals[i].type, locals[i].name); }
 
         // Evaluate arrays as char*?
         static wxRegEx reConstArr(wxT("(const )?[ ]*(w)?char(_t)? *[\\[0-9\\]]*"));
@@ -333,7 +322,6 @@ void LocalsTable::DoUpdateLocals(const LocalVariables& localsUnSorted, size_t ki
                 DbgTreeItemData* data = new DbgTreeItemData();
                 data->_kind = DbgTreeItemData::VariableObject;
                 wxTreeItemId item = m_listTable->AppendItem(root, newVarName, -1, -1, data);
-                // m_listTable->SetItemBackgroundColour(item, rootItemColour);
 
                 m_listTable->AppendItem(item, wxT("<dummy>"));
                 m_listTable->Collapse(item);
@@ -351,12 +339,12 @@ void LocalsTable::DoUpdateLocals(const LocalVariables& localsUnSorted, size_t ki
                 wxTreeItemId item = m_listTable->AppendItem(root, locals[i].name, -1, -1, new DbgTreeItemData());
                 // m_listTable->SetItemTextColour(item, DrawingUtils::GetMenuTextColour());
 
-                m_listTable->SetItemText(item, 1, locals[i].value);
-                m_listTable->SetItemText(item, 2, locals[i].type);
+                m_listTable->SetItemText(item, locals[i].value, 1);
+                m_listTable->SetItemText(item, locals[i].type, 2);
 
                 std::map<wxString, wxString>::iterator iter = oldValues.find(locals[i].name);
                 if(iter != oldValues.end() && iter->second != locals[i].value) {
-                    m_listTable->SetItemTextColour(item, *wxRED);
+                    m_listTable->SetItemTextColour(item, *wxRED, 1);
                 }
 
                 m_listTable->AppendItem(item, wxT("<dummy>"));
@@ -439,15 +427,11 @@ void LocalsTable::UpdateFuncReturnValue(const wxString& retValueGdbId)
     wxTreeItemId root = m_listTable->GetRootItem();
     if(!root.IsOk()) return;
 
-    wxColour rootItemColour = DrawingUtils::LightColour(wxT("LIGHT GRAY"), 3.0);
-
     wxArrayString itemsNotRemoved;
     // remove the non-variable objects and return a list
     // of all the variable objects (at the top level)
     wxTreeItemId item = DoFindItemByExpression(wxT("Function Returned"));
-    if(item.IsOk()) {
-        DoDeleteWatch(item);
-    }
+    if(item.IsOk()) { DoDeleteWatch(item); }
     m_listTable->Delete(item);
 
     DbgTreeItemData* data = new DbgTreeItemData();
@@ -455,11 +439,10 @@ void LocalsTable::UpdateFuncReturnValue(const wxString& retValueGdbId)
     data->_retValueGdbValue = retValueGdbId;
 
     item = m_listTable->AppendItem(root, wxT("Function Returned"), -1, -1, data);
-    m_listTable->SetItemBackgroundColour(item, rootItemColour);
 
-    m_listTable->SetItemText(item, 1, retValueGdbId);
-    m_listTable->SetItemText(item, 2, wxT(""));
-    m_listTable->SetItemTextColour(item, *wxRED);
+    m_listTable->SetItemText(item, retValueGdbId, 1);
+    m_listTable->SetItemText(item, wxT(""), 2);
+    m_listTable->SetItemTextColour(item, *wxRED, 1);
 
     m_listTable->AppendItem(item, wxT("<dummy>"));
     m_listTable->Collapse(item);
@@ -484,5 +467,20 @@ void LocalsTable::OnSortItems(wxCommandEvent& event)
 {
     // Change the sorting type and refresh the view
     m_sortAsc = !m_sortAsc;
+    // Update the sorting function
+    SetSortingFunction();
     OnRefresh(event);
 }
+
+void LocalsTable::SetSortingFunction()
+{
+    // Should a be placed before b?
+    clSortFunc_t func = [=](clRowEntry* a, clRowEntry* b) {
+        if(m_sortAsc) {
+            return (a->GetLabel().CmpNoCase(b->GetLabel()) < 0);
+        } else {
+            return (a->GetLabel().CmpNoCase(b->GetLabel()) > 0);
+        }
+    };
+    m_listTable->SetSortFunction(func);
+}
