@@ -44,6 +44,16 @@
 #include <wx/combobox.h>
 #include <wx/sizer.h>
 #include <wx/xrc/xmlres.h>
+#include <wx/dcbuffer.h>
+#include "drawingutils.h"
+#include <wx/dcbuffer.h>
+
+static bool SortPinnedProjects(clRowEntry* a, clRowEntry* b)
+{
+    wxString a_label = a->GetLabel(0);
+    wxString b_label = b->GetLabel(0);
+    return a_label.CmpNoCase(b_label) < 0;
+}
 
 WorkspaceTab::WorkspaceTab(wxWindow* parent, const wxString& caption)
     : WorkspaceTabBase(parent)
@@ -52,6 +62,9 @@ WorkspaceTab::WorkspaceTab(wxWindow* parent, const wxString& caption)
     , m_dlg(NULL)
     , m_view(NULL)
 {
+    SetBackgroundStyle(wxBG_STYLE_PAINT);
+    m_panelCxx->SetBackgroundStyle(wxBG_STYLE_PAINT);
+
     long link = EditorConfigST::Get()->GetInteger(wxT("LinkWorkspaceViewToEditor"), 1);
     m_isLinkedToEditor = link ? true : false;
 
@@ -61,6 +74,25 @@ WorkspaceTab::WorkspaceTab(wxWindow* parent, const wxString& caption)
     SetDropTarget(new clFileOrFolderDropTarget(this));
     Bind(wxEVT_DND_FOLDER_DROPPED, &WorkspaceTab::OnFolderDropped, this);
     EventNotifier::Get()->Bind(wxEVT_WORKSPACE_BUILD_CONFIG_CHANGED, &WorkspaceTab::OnConfigChanged, this);
+
+    m_panelCxx->Bind(wxEVT_PAINT, [&](wxPaintEvent& e) {
+        wxAutoBufferedPaintDC dc(m_panelCxx);
+        dc.SetBrush(m_bgColour);
+        dc.SetPen(m_bgColour);
+        dc.DrawRectangle(m_panelCxx->GetClientRect());
+    });
+
+    m_bgColour = DrawingUtils::GetPanelBgColour();
+    EventNotifier::Get()->Bind(wxEVT_CMD_COLOURS_FONTS_UPDATED, [&](clCommandEvent& event) {
+        event.Skip();
+        bool useCustomColour = clConfig::Get().Read("UseCustomBaseColour", false);
+        m_bgColour = DrawingUtils::GetPanelBgColour();
+        if(useCustomColour) { m_bgColour = clConfig::Get().Read("BaseColour", m_bgColour); }
+        Refresh();
+        m_panelCxx->Refresh();
+    });
+    m_bitmaps.push_back(clGetManager()->GetStdIcons()->LoadBitmap("project"));
+    m_dvListCtrlPinnedProjects->SetBitmaps(&m_bitmaps);
 }
 
 WorkspaceTab::~WorkspaceTab()
@@ -92,20 +124,46 @@ WorkspaceTab::~WorkspaceTab()
     wxTheApp->Disconnect(XRCID("configuration_manager"), wxEVT_UPDATE_UI,
                          wxUpdateUIEventHandler(WorkspaceTab::OnProjectSettingsUI), NULL, this);
     EventNotifier::Get()->Unbind(wxEVT_WORKSPACE_BUILD_CONFIG_CHANGED, &WorkspaceTab::OnConfigChanged, this);
+    m_toolbar580->Unbind(wxEVT_TOOL, &WorkspaceTab::OnCollapseAll, this, XRCID("ID_TOOL_COLLAPSE_ALL"));
+    m_toolbar580->Unbind(wxEVT_UPDATE_UI, &WorkspaceTab::OnCollapseAllUI, this, XRCID("ID_TOOL_COLLAPSE_ALL"));
+    m_toolbar580->Unbind(wxEVT_TOOL, &WorkspaceTab::OnLinkEditor, this, XRCID("ID_TOOL_LINK_EDITOR"));
+    m_toolbar580->Unbind(wxEVT_UPDATE_UI, &WorkspaceTab::OnLinkEditorUI, this, XRCID("ID_TOOL_LINK_EDITOR"));
+    m_toolbar580->Unbind(wxEVT_TOOL, &WorkspaceTab::OnProjectSettings, this, XRCID("ID_TOOL_ACTIVE_PROJECT_SETTINGS"));
+    m_toolbar580->Unbind(wxEVT_UPDATE_UI, &WorkspaceTab::OnProjectSettingsUI, this,
+                         XRCID("ID_TOOL_ACTIVE_PROJECT_SETTINGS"));
+    m_toolbar580->Unbind(wxEVT_TOOL, &WorkspaceTab::OnGoHome, this, XRCID("ID_TOOL_GOTO_ACTIVE_PROJECT"));
+    m_toolbar580->Unbind(wxEVT_UPDATE_UI, &WorkspaceTab::OnGoHomeUI, this, XRCID("ID_TOOL_GOTO_ACTIVE_PROJECT"));
 }
 
 void WorkspaceTab::CreateGUIControls()
 {
-    //// Construct the tree
     int index = m_simpleBook->FindPage(m_panelCxx);
     if(index != wxNOT_FOUND) {
         // set the C++ workspace view name to fit its workspace type
+        //// Construct the tree
         m_simpleBook->SetPageText(index, clCxxWorkspaceST::Get()->GetWorkspaceType());
     }
     m_view = new clWorkspaceView(m_simpleBook);
     m_view->AddPage(new DefaultWorkspacePage(m_simpleBook), _("Default"));
     m_view->SelectPage(_("Default"));
     m_view->SetDefaultPage(_("Default"));
+    BitmapLoader* bmps = clGetManager()->GetStdIcons();
+    m_toolbar580->AddTool(XRCID("ID_TOOL_LINK_EDITOR"), _("Link Editor"), bmps->LoadBitmap("link_editor"), "",
+                          wxITEM_CHECK);
+    m_toolbar580->AddTool(XRCID("ID_TOOL_COLLAPSE_ALL"), _("Collapse All"), bmps->LoadBitmap("fold"));
+    m_toolbar580->AddTool(XRCID("ID_TOOL_GOTO_ACTIVE_PROJECT"), _("Goto Active Project"), bmps->LoadBitmap("home"));
+    m_toolbar580->AddTool(XRCID("ID_TOOL_ACTIVE_PROJECT_SETTINGS"),
+                          _("Open selected project settings. If there is no project selected, open the parent project "
+                            "of the selected item in the tree"),
+                          bmps->LoadBitmap("cog"));
+    m_toolbar580->AddSpacer();
+    m_toolbar580->AddTool(XRCID("execute_no_debug"), _("Run Active Project"), bmps->LoadBitmap("execute"),
+                          _("Run Active Project"));
+    m_toolbar580->AddTool(XRCID("build_active_project"), _("Build Active Project"), bmps->LoadBitmap("build"),
+                          _("Build Active Project"), wxITEM_DROPDOWN);
+    m_toolbar580->AddTool(XRCID("stop_active_project_build"), _("Stop Current Build"), bmps->LoadBitmap("stop"),
+                          _("Stop Current Build"));
+    m_toolbar580->Realize();
 }
 
 void WorkspaceTab::FreezeThaw(bool freeze /*=true*/)
@@ -154,11 +212,20 @@ void WorkspaceTab::ConnectEvents()
                       wxCommandEventHandler(WorkspaceTab::OnConfigurationManager), NULL, this);
     wxTheApp->Connect(XRCID("configuration_manager"), wxEVT_UPDATE_UI,
                       wxUpdateUIEventHandler(WorkspaceTab::OnProjectSettingsUI), NULL, this);
+    m_toolbar580->Bind(wxEVT_TOOL, &WorkspaceTab::OnCollapseAll, this, XRCID("ID_TOOL_COLLAPSE_ALL"));
+    m_toolbar580->Bind(wxEVT_UPDATE_UI, &WorkspaceTab::OnCollapseAllUI, this, XRCID("ID_TOOL_COLLAPSE_ALL"));
+    m_toolbar580->Bind(wxEVT_TOOL, &WorkspaceTab::OnLinkEditor, this, XRCID("ID_TOOL_LINK_EDITOR"));
+    m_toolbar580->Bind(wxEVT_UPDATE_UI, &WorkspaceTab::OnLinkEditorUI, this, XRCID("ID_TOOL_LINK_EDITOR"));
+    m_toolbar580->Bind(wxEVT_TOOL, &WorkspaceTab::OnProjectSettings, this, XRCID("ID_TOOL_ACTIVE_PROJECT_SETTINGS"));
+    m_toolbar580->Bind(wxEVT_UPDATE_UI, &WorkspaceTab::OnProjectSettingsUI, this,
+                       XRCID("ID_TOOL_ACTIVE_PROJECT_SETTINGS"));
+    m_toolbar580->Bind(wxEVT_TOOL, &WorkspaceTab::OnGoHome, this, XRCID("ID_TOOL_GOTO_ACTIVE_PROJECT"));
+    m_toolbar580->Bind(wxEVT_UPDATE_UI, &WorkspaceTab::OnGoHomeUI, this, XRCID("ID_TOOL_GOTO_ACTIVE_PROJECT"));
 }
 
 void WorkspaceTab::OnLinkEditor(wxCommandEvent& e)
 {
-    m_isLinkedToEditor = !m_isLinkedToEditor;
+    m_isLinkedToEditor = e.IsChecked();
     EditorConfigST::Get()->SetInteger(wxT("LinkWorkspaceViewToEditor"), m_isLinkedToEditor ? 1 : 0);
     if(m_isLinkedToEditor) { OnActiveEditorChanged(e); }
 }
@@ -167,16 +234,16 @@ void WorkspaceTab::OnCollapseAll(wxCommandEvent& e)
 {
     wxUnusedVar(e);
     if(!m_fileView->GetRootItem().IsOk()) return;
-    m_fileView->Freeze();
     m_fileView->CollapseAll();
-    m_fileView->Expand(m_fileView->GetRootItem());
+
     // count will probably be 0 below, so ensure we can at least see the root item
     m_fileView->EnsureVisible(m_fileView->GetRootItem());
-    m_fileView->Thaw();
+    m_fileView->SelectItem(m_fileView->GetRootItem());
+    // Expand the workspace
+    m_fileView->Expand(m_fileView->GetRootItem());
 
     wxArrayTreeItemIds arr;
     size_t count = m_fileView->GetSelections(arr);
-
     if(count == 1) {
         wxTreeItemId sel = arr.Item(0);
         if(sel.IsOk()) { m_fileView->EnsureVisible(sel); }
@@ -218,7 +285,7 @@ void WorkspaceTab::OnProjectSettingsUI(wxUpdateUIEvent& e)
 
 void WorkspaceTab::OnShowFile(wxCommandEvent& e)
 {
-    LEditor* editor = clMainFrame::Get()->GetMainBook()->GetActiveEditor();
+    clEditor* editor = clMainFrame::Get()->GetMainBook()->GetActiveEditor();
     if(editor && !editor->GetProject().IsEmpty()) {
         m_fileView->ExpandToPath(editor->GetProject(), editor->GetFileName());
         ManagerST::Get()->ShowWorkspacePane(m_caption);
@@ -228,7 +295,7 @@ void WorkspaceTab::OnShowFile(wxCommandEvent& e)
 
 void WorkspaceTab::OnShowFileUI(wxUpdateUIEvent& e)
 {
-    LEditor* editor = clMainFrame::Get()->GetMainBook()->GetActiveEditor();
+    clEditor* editor = clMainFrame::Get()->GetMainBook()->GetActiveEditor();
     e.Enable(editor && !editor->GetProject().IsEmpty());
 }
 
@@ -237,7 +304,7 @@ void WorkspaceTab::OnActiveEditorChanged(wxCommandEvent& e)
     e.Skip();
     if(m_isLinkedToEditor) {
         MainBook* mainbook = clMainFrame::Get()->GetMainBook();
-        LEditor* editor = mainbook->GetActiveEditor();
+        clEditor* editor = mainbook->GetActiveEditor();
         if(editor && !editor->GetProject().IsEmpty()) {
             m_fileView->ExpandToPath(editor->GetProject(), editor->GetFileName());
         }
@@ -264,6 +331,9 @@ void WorkspaceTab::OnWorkspaceLoaded(wxCommandEvent& e)
         m_fileView->BuildTree();
         CallAfter(&WorkspaceTab::DoGoHome);
         SendCmdEvent(wxEVT_FILE_VIEW_INIT_DONE);
+
+        // Load the C++ pinned projects list
+        LoadCxxPinnedProjects();
     }
 }
 
@@ -275,6 +345,12 @@ void WorkspaceTab::OnWorkspaceClosed(wxCommandEvent& e)
     m_configChangeCtrl->Clear();
     m_configChangeCtrl->Enable(false);
     m_fileView->DeleteAllItems();
+
+    // Clear the pinned projects view
+    m_dvListCtrlPinnedProjects->DeleteAllItems();
+    SaveCxxPinnedProjects();
+    m_cxxPinnedProjects.clear();
+
     SendCmdEvent(wxEVT_FILE_VIEW_INIT_DONE);
 }
 
@@ -429,7 +505,7 @@ void WorkspaceTab::DoConfigChanged(const wxString& newConfigName)
     ManagerST::Get()->SetWorkspaceBuildMatrix(matrix);
 
     // Set the focus to the active editor if any
-    LEditor* editor = clMainFrame::Get()->GetMainBook()->GetActiveEditor();
+    clEditor* editor = clMainFrame::Get()->GetMainBook()->GetActiveEditor();
     if(editor) { editor->SetActive(); }
 
     ManagerST::Get()->UpdateParserPaths(true);
@@ -454,3 +530,108 @@ void WorkspaceTab::OnConfigChanged(clCommandEvent& e)
     }
     DoConfigChanged(selection);
 }
+
+void WorkspaceTab::OnPaint(wxPaintEvent& event)
+{
+    wxAutoBufferedPaintDC dc(this);
+    dc.SetBrush(m_bgColour);
+    dc.SetPen(m_bgColour);
+    dc.DrawRectangle(GetClientRect());
+}
+
+void WorkspaceTab::LoadCxxPinnedProjects()
+{
+    m_cxxPinnedProjects.clear();
+    clCxxWorkspaceST::Get()->GetLocalWorkspace()->GetPinnedProjects(m_cxxPinnedProjects);
+
+    // We got the pinned projects loaded, update the view
+    if(m_cxxPinnedProjects.empty()) {
+        // hide the to view
+        if(m_splitter->IsSplit()) { m_splitter->Unsplit(m_splitterPagePinnedProjects); }
+    } else {
+        // ensure the view is visible
+        if(!m_splitter->IsSplit()) {
+            m_splitter->SplitHorizontally(m_splitterPagePinnedProjects, m_splitterPageTreeView, 150);
+            m_splitter->CallAfter(&wxSplitterWindow::UpdateSize);
+        }
+        
+        m_dvListCtrlPinnedProjects->DeleteAllItems();
+        m_dvListCtrlPinnedProjects->SetSortFunction(nullptr);
+        for(const wxString& project : m_cxxPinnedProjects) {
+            wxVector<wxVariant> V;
+            V.push_back(::MakeBitmapIndexText(project, 0));
+            m_dvListCtrlPinnedProjects->AppendItem(V);
+        }
+        // apply the sort
+        m_dvListCtrlPinnedProjects->SetSortFunction(SortPinnedProjects);
+    }
+}
+
+void WorkspaceTab::SaveCxxPinnedProjects()
+{
+    clCxxWorkspaceST::Get()->GetLocalWorkspace()->SetPinnedProjects(m_cxxPinnedProjects);
+}
+
+void WorkspaceTab::OnPinnedCxxProjectContextMenu(wxDataViewEvent& event)
+{
+    wxDataViewItem item = event.GetItem();
+    if(!item.IsOk()) { return; }
+    SyncPinnedProjectsView(event.GetItem());
+
+    wxString project = m_dvListCtrlPinnedProjects->GetItemText(item);
+    ShowPinnedProjectMenu(project);
+}
+
+void WorkspaceTab::OnPinnedCxxProjectSelected(wxDataViewEvent& event)
+{
+    // we MUST sync the views, otherwise, the context menu wont work
+    // since it reliese on the current selection in the tree
+    SyncPinnedProjectsView(event.GetItem());
+}
+
+void WorkspaceTab::AddPinnedProject(const wxString& project)
+{
+    if(m_cxxPinnedProjects.Index(project) != wxNOT_FOUND) { return; }
+    m_cxxPinnedProjects.Add(project);
+
+    // Store the pinned projehcts
+    SaveCxxPinnedProjects();
+
+    // Refresh the view
+    LoadCxxPinnedProjects();
+}
+
+void WorkspaceTab::SyncPinnedProjectsView(const wxDataViewItem& item)
+{
+    if(!item.IsOk()) { return; }
+
+    wxString project = m_dvListCtrlPinnedProjects->GetItemText(item);
+    m_fileView->ExpandToPath(project, wxFileName()); // Select the project
+}
+
+void WorkspaceTab::ShowPinnedProjectMenu(const wxString& project)
+{
+    wxMenu menu;
+    menu.Append(XRCID("unpin_project"), _("Unpin Project"));
+    menu.AppendSeparator();
+    // Build the rest of the menu
+    m_fileView->CreateProjectContextMenu(menu, project, false);
+    menu.Bind(wxEVT_MENU, [&](wxCommandEvent& menuEvent) {
+        if(menuEvent.GetId() == XRCID("unpin_project")) {
+            wxDataViewItem item = m_dvListCtrlPinnedProjects->GetSelection();
+            if(item.IsOk()) {
+                m_dvListCtrlPinnedProjects->DeleteItem(m_dvListCtrlPinnedProjects->ItemToRow(item));
+                int where = m_cxxPinnedProjects.Index(project);
+                if(where != wxNOT_FOUND) {
+                    m_cxxPinnedProjects.RemoveAt(where);
+                    SaveCxxPinnedProjects();
+                    CallAfter(&WorkspaceTab::LoadCxxPinnedProjects);
+                }
+            }
+        } else {
+            // Pass all the context menu events to the tree tree view
+            m_fileView->GetEventHandler()->ProcessEvent(menuEvent);
+        }
+    });
+    m_dvListCtrlPinnedProjects->PopupMenu(&menu);
+}
