@@ -8,7 +8,7 @@
 #include "fileextmanager.h"
 #include "fileutils.h"
 #include "globals.h"
-#include "json_node.h"
+#include "JSON.h"
 #include "macros.h"
 #include "wxStringHash.h"
 #include "xmlutils.h"
@@ -22,6 +22,10 @@
 #include <wx/sstream.h>
 #include <wx/tokenzr.h>
 #include <wx/xml/xml.h>
+#include "globals.h"
+#include <imanager.h>
+#include <wx/xrc/xmlres.h>
+#include "theme_handler_helper.h"
 
 // Upgrade macros
 #define LEXERS_VERSION_STRING "LexersVersion"
@@ -94,13 +98,15 @@ ColoursAndFontsManager::ColoursAndFontsManager()
         m_defaultLexer.Reset(new LexerConf());
         m_defaultLexer->FromXml(doc.GetRoot());
     }
-
-    m_globalBgColour = wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW);
-    m_globalFgColour = wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT);
     m_lexersVersion = clConfig::Get().Read(LEXERS_VERSION_STRING, LEXERS_UPGRADE_LINENUM_DEFAULT_COLOURS);
+    EventNotifier::Get()->Bind(wxEVT_INFO_BAR_BUTTON, &ColoursAndFontsManager::OnAdjustTheme, this);
 }
 
-ColoursAndFontsManager::~ColoursAndFontsManager() { clConfig::Get().Write(LEXERS_VERSION_STRING, LEXERS_VERSION); }
+ColoursAndFontsManager::~ColoursAndFontsManager()
+{
+    clConfig::Get().Write(LEXERS_VERSION_STRING, LEXERS_VERSION);
+    EventNotifier::Get()->Unbind(wxEVT_INFO_BAR_BUTTON, &ColoursAndFontsManager::OnAdjustTheme, this);
+}
 
 ColoursAndFontsManager& ColoursAndFontsManager::Get()
 {
@@ -164,12 +170,8 @@ void ColoursAndFontsManager::Load()
 
     // Load the global settings
     if(GetConfigFile().FileExists()) {
-        JSONRoot root(GetConfigFile());
-        if(root.isOk()) {
-            m_globalBgColour = root.toElement().namedObject("m_globalBgColour").toColour(m_globalBgColour);
-            m_globalFgColour = root.toElement().namedObject("m_globalFgColour").toColour(m_globalFgColour);
-            m_globalTheme = root.toElement().namedObject("m_globalTheme").toString("Default");
-        }
+        JSON root(GetConfigFile());
+        if(root.isOk()) { m_globalTheme = root.toElement().namedObject("m_globalTheme").toString("Default"); }
     }
 
     // Load the lexers
@@ -212,9 +214,7 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(wxXmlNode* node)
 
     // Hack: fix Java lexer which is using the same
     // file extensions as C++...
-    if(lexer->GetName() == "java" && lexer->GetFileSpec().Contains(".cpp")) {
-        lexer->SetFileSpec("*.java");
-    }
+    if(lexer->GetName() == "java" && lexer->GetFileSpec().Contains(".cpp")) { lexer->SetFileSpec("*.java"); }
 
     // Append *.sqlite to the SQL lexer if missing
     if(lexer->GetName() == "sql" && !lexer->GetFileSpec().Contains(".sqlite")) {
@@ -250,6 +250,22 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(wxXmlNode* node)
         lexer->SetFileSpec(lexer->GetFileSpec() + ";*.wxcp");
     }
 
+    if(lexer->GetName() == "javascript") {
+        wxString jsWords = lexer->GetKeyWords(0);
+        wxArrayString arrWords = ::wxStringTokenize(jsWords, " ", wxTOKEN_STRTOK);
+        // use std::set to make sure that the elements are sorted
+        std::set<wxString> uniqueSet;
+        for(size_t i = 0; i < arrWords.size(); ++i) {
+            uniqueSet.insert(arrWords.Item(i));
+        }
+        uniqueSet.insert("class");
+        uniqueSet.insert("await");
+        uniqueSet.insert("async");
+        jsWords.clear();
+        std::for_each(uniqueSet.begin(), uniqueSet.end(), [&](const wxString& word) { jsWords << word << " "; });
+        lexer->SetKeyWords(jsWords, 0);
+    }
+
     // Add *.scss file extension to the css lexer
     if(lexer->GetName() == "css" && !lexer->GetFileSpec().Contains(".scss")) {
         lexer->SetFileSpec(lexer->GetFileSpec() + ";*.scss");
@@ -274,6 +290,10 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(wxXmlNode* node)
             1);
     }
 
+    if(lexer->GetName() == "makefile" && !lexer->GetFileSpec().Contains("*akefile.am")) {
+        lexer->SetFileSpec(lexer->GetFileSpec() + ";*akefile.in;*akefile.am");
+    }
+
     // Upgrade the lexer colours
     UpdateLexerColours(lexer, false);
 
@@ -288,14 +308,10 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(wxXmlNode* node)
     // name
     ColoursAndFontsManager::Vec_t::iterator iter =
         std::find_if(vec.begin(), vec.end(), LexerConf::FindByNameAndTheme(lexer->GetName(), lexer->GetThemeName()));
-    if(iter != vec.end()) {
-        vec.erase(iter);
-    }
+    if(iter != vec.end()) { vec.erase(iter); }
     iter = std::find_if(m_allLexers.begin(), m_allLexers.end(),
                         LexerConf::FindByNameAndTheme(lexer->GetName(), lexer->GetThemeName()));
-    if(iter != m_allLexers.end()) {
-        m_allLexers.erase(iter);
-    }
+    if(iter != m_allLexers.end()) { m_allLexers.erase(iter); }
     vec.push_back(lexer);
     m_allLexers.push_back(lexer);
     return lexer;
@@ -331,13 +347,9 @@ LexerConf::Ptr_t ColoursAndFontsManager::GetLexer(const wxString& lexerName, con
         const ColoursAndFontsManager::Vec_t& lexers = iter->second;
         for(size_t i = 0; i < lexers.size(); ++i) {
 
-            if(!firstLexer) {
-                firstLexer = lexers.at(i);
-            }
+            if(!firstLexer) { firstLexer = lexers.at(i); }
 
-            if(!defaultLexer && lexers.at(i)->GetThemeName() == "Default") {
-                defaultLexer = lexers.at(i);
-            }
+            if(!defaultLexer && lexers.at(i)->GetThemeName() == "Default") { defaultLexer = lexers.at(i); }
 
             if(lexers.at(i)->IsActive()) return lexers.at(i);
         }
@@ -352,24 +364,26 @@ LexerConf::Ptr_t ColoursAndFontsManager::GetLexer(const wxString& lexerName, con
 
     } else {
         const ColoursAndFontsManager::Vec_t& lexers = iter->second;
+        LexerConf::Ptr_t themeDefaultLexer = nullptr;
         for(size_t i = 0; i < lexers.size(); ++i) {
-            if(lexers.at(i)->GetThemeName() == theme) {
-                return lexers.at(i);
-            }
+            if(lexers[i]->GetThemeName() == "Default") { themeDefaultLexer = lexers[i]; }
+            if(lexers[i]->GetThemeName() == theme) { return lexers[i]; }
         }
-        return m_defaultLexer;
+        // We failed to find the requested theme for this language. If we have a "Default"
+        // lexer, return it, else use the minimal lexer ("m_defaultLexer")
+        return (themeDefaultLexer ? themeDefaultLexer : m_defaultLexer);
     }
 }
 
-void ColoursAndFontsManager::Save()
+void ColoursAndFontsManager::Save(bool forExport)
 {
     ColoursAndFontsManager::Map_t::const_iterator iter = m_lexersMap.begin();
-    JSONRoot root(cJSON_Array);
-    JSONElement element = root.toElement();
+    JSON root(cJSON_Array);
+    JSONItem element = root.toElement();
     for(; iter != m_lexersMap.end(); ++iter) {
         const ColoursAndFontsManager::Vec_t& lexers = iter->second;
         for(size_t i = 0; i < lexers.size(); ++i) {
-            element.arrayAppend(lexers.at(i)->ToJSON());
+            element.arrayAppend(lexers.at(i)->ToJSON(forExport));
         }
     }
 
@@ -387,9 +401,7 @@ wxArrayString ColoursAndFontsManager::GetAllLexersNames() const
     wxArrayString names;
     for(size_t i = 0; i < m_allLexers.size(); ++i) {
         LexerConf::Ptr_t lexer = m_allLexers.at(i);
-        if(names.Index(lexer->GetName()) == wxNOT_FOUND) {
-            names.Add(lexer->GetName());
-        }
+        if(names.Index(lexer->GetName()) == wxNOT_FOUND) { names.Add(lexer->GetName()); }
     }
     names.Sort();
     return names;
@@ -427,9 +439,7 @@ LexerConf::Ptr_t ColoursAndFontsManager::GetLexerForFile(const wxString& filenam
     // the file mask. However, if we did find a "firstLexer" it means
     // that we do have a lexer that matches the file extension, its just that it is not
     // set as active
-    if(firstLexer) {
-        return firstLexer;
-    }
+    if(firstLexer) { return firstLexer; }
 
     // Try this:
     // Use the FileExtManager to get the file type by examinig its content
@@ -440,6 +450,9 @@ LexerConf::Ptr_t ColoursAndFontsManager::GetLexerForFile(const wxString& filenam
         case FileExtManager::TypeScript:
             lexerByContent = GetLexer("script");
             break;
+        case FileExtManager::TypeJS:
+            lexerByContent = GetLexer("javascript");
+            break;
         case FileExtManager::TypePhp:
             lexerByContent = GetLexer("php");
             break;
@@ -491,9 +504,7 @@ void ColoursAndFontsManager::SetActiveTheme(const wxString& lexerName, const wxS
     wxArrayString themes = GetAvailableThemesForLexer(lexerName);
     for(size_t i = 0; i < themes.GetCount(); ++i) {
         LexerConf::Ptr_t lexer = GetLexer(lexerName, themes.Item(i));
-        if(lexer && lexer->GetName() == lexerName) {
-            lexer->SetIsActive(lexer->GetThemeName() == themeName);
-        }
+        if(lexer && lexer->GetName() == lexerName) { lexer->SetIsActive(lexer->GetThemeName() == themeName); }
     }
 }
 
@@ -507,11 +518,8 @@ wxFileName ColoursAndFontsManager::GetConfigFile() const
 void ColoursAndFontsManager::SaveGlobalSettings()
 {
     // save the global settings
-    JSONRoot root(cJSON_Object);
-    root.toElement()
-        .addProperty("m_globalBgColour", m_globalBgColour)
-        .addProperty("m_globalFgColour", m_globalFgColour)
-        .addProperty("m_globalTheme", m_globalTheme);
+    JSON root(cJSON_Object);
+    root.toElement().addProperty("m_globalTheme", m_globalTheme);
     wxFileName fnSettings = GetConfigFile();
     root.save(fnSettings.GetFullPath());
 
@@ -525,7 +533,7 @@ LexerConf::Ptr_t ColoursAndFontsManager::CopyTheme(const wxString& lexerName, co
     LexerConf::Ptr_t sourceLexer = GetLexer(lexerName, sourceTheme);
     CHECK_PTR_RET_NULL(sourceLexer);
 
-    JSONElement json = sourceLexer->ToJSON();
+    JSONItem json = sourceLexer->ToJSON();
     LexerConf::Ptr_t newLexer(new LexerConf());
     newLexer->FromJSON(json);
 
@@ -542,9 +550,7 @@ void ColoursAndFontsManager::RestoreDefaults()
     {
         wxLogNull noLog;
         wxFileName fnLexersJSON(clStandardPaths::Get().GetUserLexersDir(), "lexers.json");
-        if(fnLexersJSON.Exists()) {
-            clRemoveFile(fnLexersJSON.GetFullPath());
-        }
+        if(fnLexersJSON.Exists()) { clRemoveFile(fnLexersJSON.GetFullPath()); }
     }
 
     // Now, we simply reload the settings
@@ -619,9 +625,7 @@ void ColoursAndFontsManager::UpdateLexerColours(LexerConf::Ptr_t lexer, bool for
         } else {
             lexer->SetLineNumbersFgColour(wxSystemSettings::GetColour(wxSYS_COLOUR_GRAYTEXT));
             StyleProperty& lineNumbers = lexer->GetProperty(LINE_NUMBERS_ATTR_ID); // Line numbers
-            if(!lineNumbers.IsNull()) {
-                lineNumbers.SetBgColour(defaultProp.GetBgColour());
-            }
+            if(!lineNumbers.IsNull()) { lineNumbers.SetBgColour(defaultProp.GetBgColour()); }
 
             // don't adjust PHP and HTML default colours, since they also affects the various operators
             // foreground colours
@@ -684,18 +688,10 @@ void ColoursAndFontsManager::UpdateLexerColours(LexerConf::Ptr_t lexer, bool for
         StyleProperty& identifier3 = lexer->GetProperty(wxSTC_CSS_IDENTIFIER3);
         StyleProperty& oper = lexer->GetProperty(wxSTC_CSS_OPERATOR);
         if(!var.IsNull()) {
-            if(!identifier.IsNull()) {
-                identifier.SetFgColour(var.GetFgColour());
-            }
-            if(!identifier2.IsNull()) {
-                identifier2.SetFgColour(var.GetFgColour());
-            }
-            if(!identifier3.IsNull()) {
-                identifier3.SetFgColour(var.GetFgColour());
-            }
-            if(!oper.IsNull()) {
-                oper.SetFgColour(isDark ? "WHITE" : "BLACK");
-            }
+            if(!identifier.IsNull()) { identifier.SetFgColour(var.GetFgColour()); }
+            if(!identifier2.IsNull()) { identifier2.SetFgColour(var.GetFgColour()); }
+            if(!identifier3.IsNull()) { identifier3.SetFgColour(var.GetFgColour()); }
+            if(!oper.IsNull()) { oper.SetFgColour(isDark ? "WHITE" : "BLACK"); }
         }
     }
 
@@ -717,7 +713,7 @@ void ColoursAndFontsManager::SetTheme(const wxString& themeName)
     if(isDark) {
         fallbackTheme = "One Dark Like";
     } else {
-        fallbackTheme = "Default";
+        fallbackTheme = "Atom One Light";
     }
 
     const wxArrayString& lexers = GetAllLexersNames();
@@ -730,24 +726,31 @@ void ColoursAndFontsManager::SetTheme(const wxString& themeName)
         }
     }
     SetGlobalTheme(themeName);
+    
+    clColours colours;
+    bool useCustomColour = clConfig::Get().Read("UseCustomBaseColour", false);
+    if(useCustomColour) {
+        wxColour bgColour = GetBackgroundColourFromLexer(lexer);
+        clConfig::Get().Write("BaseColour", bgColour);
+    }
 }
 
 void ColoursAndFontsManager::LoadJSON(const wxFileName& path)
 {
     if(!path.FileExists()) return;
 
-    JSONRoot root(path);
-    JSONElement arr = root.toElement();
+    JSON root(path);
+    JSONItem arr = root.toElement();
     int arrSize = arr.arraySize();
     CL_DEBUG("Loading JSON file: %s (contains %d lexers)", path.GetFullPath(), arrSize);
     for(int i = 0; i < arrSize; ++i) {
-        JSONElement json = arr.arrayItem(i);
+        JSONItem json = arr.arrayItem(i);
         DoAddLexer(json);
     }
     CL_DEBUG("Loading JSON file...done");
 }
 
-LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(JSONElement json)
+LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(JSONItem json)
 {
     LexerConf::Ptr_t lexer(new LexerConf());
     lexer->FromJSON(json);
@@ -761,7 +764,7 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(JSONElement json)
     themeName = themeName.Mid(0, 1).Capitalize() + themeName.Mid(1);
     lexer->SetThemeName(themeName);
 
-    CL_DEBUG("Loading lexer: %s [%s]", lexerName, lexer->GetName());
+    clDEBUG1() << "Loading lexer:" << lexerName;
 
     if(lexer->GetName() == "c++" && !lexer->GetKeyWords(0).Contains("final")) {
         lexer->SetKeyWords(lexer->GetKeyWords(0) + " final", 0);
@@ -773,9 +776,7 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(JSONElement json)
 
     // Hack: fix Java lexer which is using the same
     // file extensions as C++...
-    if(lexer->GetName() == "java" && lexer->GetFileSpec().Contains(".cpp")) {
-        lexer->SetFileSpec("*.java");
-    }
+    if(lexer->GetName() == "java" && lexer->GetFileSpec().Contains(".cpp")) { lexer->SetFileSpec("*.java"); }
 
     // Append *.sqlite to the SQL lexer if missing
     if(lexer->GetName() == "sql" && !lexer->GetFileSpec().Contains(".sqlite")) {
@@ -799,9 +800,7 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(JSONElement json)
     }
 
     // Hack5: all the remove *.scss from the css lexer (it now has its own lexer)
-    if(lexer->GetName() == "css" && lexer->GetFileSpec().Contains(".scss")) {
-        lexer->SetFileSpec("*.css");
-    }
+    if(lexer->GetName() == "css" && lexer->GetFileSpec().Contains(".scss")) { lexer->SetFileSpec("*.css"); }
 
     // Add *.less file extension to the css lexer
     if(lexer->GetName() == "css" && !lexer->GetFileSpec().Contains(".less")) {
@@ -828,10 +827,28 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(JSONElement json)
     if(lexer->GetName() == "javascript" && !lexer->GetFileSpec().Contains(".wxcp")) {
         lexer->SetFileSpec(lexer->GetFileSpec() + ";*.wxcp");
     }
-    if(lexer->GetName() == "text") {
-        lexer->SetFileSpec(wxEmptyString);
+
+    if(lexer->GetName() == "javascript") {
+        wxString jsWords = lexer->GetKeyWords(0);
+        wxArrayString arrWords = ::wxStringTokenize(jsWords, " ", wxTOKEN_STRTOK);
+        // use std::set to make sure that the elements are sorted
+        std::set<wxString> uniqueSet;
+        for(size_t i = 0; i < arrWords.size(); ++i) {
+            uniqueSet.insert(arrWords.Item(i));
+        }
+        uniqueSet.insert("class");
+        uniqueSet.insert("await");
+        uniqueSet.insert("async");
+        uniqueSet.insert("extends");
+        uniqueSet.insert("constructor");
+        uniqueSet.insert("super");
+        jsWords.clear();
+        std::for_each(uniqueSet.begin(), uniqueSet.end(), [&](const wxString& word) { jsWords << word << " "; });
+        lexer->SetKeyWords(jsWords, 0);
     }
 
+    if(lexer->GetName() == "text") { lexer->SetFileSpec(wxEmptyString); }
+
     // Set the JavaScript keywords
     if(lexer->GetName() == "php" && !lexer->GetKeyWords(1).Contains("instanceof")) {
         lexer->SetKeyWords(
@@ -845,6 +862,11 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(JSONElement json)
             "int short null true false",
             1);
     }
+
+    if(lexer->GetName() == "makefile" && !lexer->GetFileSpec().Contains("*akefile.am")) {
+        lexer->SetFileSpec(lexer->GetFileSpec() + ";*akefile.in;*akefile.am");
+    }
+
     // Upgrade the lexer colours
     UpdateLexerColours(lexer, false);
 
@@ -859,15 +881,11 @@ LexerConf::Ptr_t ColoursAndFontsManager::DoAddLexer(JSONElement json)
     // name
     ColoursAndFontsManager::Vec_t::iterator iter =
         std::find_if(vec.begin(), vec.end(), LexerConf::FindByNameAndTheme(lexer->GetName(), lexer->GetThemeName()));
-    if(iter != vec.end()) {
-        vec.erase(iter);
-    }
+    if(iter != vec.end()) { vec.erase(iter); }
 
     iter = std::find_if(m_allLexers.begin(), m_allLexers.end(),
                         LexerConf::FindByNameAndTheme(lexer->GetName(), lexer->GetThemeName()));
-    if(iter != m_allLexers.end()) {
-        m_allLexers.erase(iter);
-    }
+    if(iter != m_allLexers.end()) { m_allLexers.erase(iter); }
     vec.push_back(lexer);
     m_allLexers.push_back(lexer);
     return lexer;
@@ -907,21 +925,19 @@ bool ColoursAndFontsManager::ExportThemesToFile(const wxFileName& outputFile, co
         M.insert(names.Item(i).Lower());
     }
 
-    JSONRoot root(cJSON_Array);
-    JSONElement arr = root.toElement();
+    JSON root(cJSON_Array);
+    JSONItem arr = root.toElement();
     std::vector<LexerConf::Ptr_t> Lexers;
     std::for_each(m_allLexers.begin(), m_allLexers.end(), [&](LexerConf::Ptr_t lexer) {
-        if(M.empty() || M.count(lexer->GetThemeName().Lower())) {
-            Lexers.push_back(lexer);
-        }
+        if(M.empty() || M.count(lexer->GetThemeName().Lower())) { Lexers.push_back(lexer); }
     });
-    std::for_each(Lexers.begin(), Lexers.end(), [&](LexerConf::Ptr_t lexer) { arr.append(lexer->ToJSON()); });
+    std::for_each(Lexers.begin(), Lexers.end(), [&](LexerConf::Ptr_t lexer) { arr.append(lexer->ToJSON(true)); });
     return FileUtils::WriteFileContent(outputFile, root.toElement().format());
 }
 
 bool ColoursAndFontsManager::ImportLexersFile(const wxFileName& inputFile, bool prompt)
 {
-    JSONRoot root(inputFile);
+    JSON root(inputFile);
     if(!root.isOk()) {
         clWARNING() << "Invalid lexers input file:" << inputFile << clEndl;
         return false;
@@ -936,19 +952,17 @@ bool ColoursAndFontsManager::ImportLexersFile(const wxFileName& inputFile, bool
     }
 
     std::vector<LexerConf::Ptr_t> Lexers;
-    JSONElement arr = root.toElement();
+    JSONItem arr = root.toElement();
     int arrSize = arr.arraySize();
     for(int i = 0; i < arrSize; ++i) {
-        JSONElement lexerObj = arr.arrayItem(i);
+        JSONItem lexerObj = arr.arrayItem(i);
         LexerConf::Ptr_t lexer(new LexerConf());
         lexer->FromJSON(lexerObj);
         Lexers.push_back(lexer);
     }
 
     std::for_each(Lexers.begin(), Lexers.end(), [&](LexerConf::Ptr_t lexer) {
-        if(m_lexersMap.count(lexer->GetName()) == 0) {
-            m_lexersMap[lexer->GetName()] = Vec_t();
-        }
+        if(m_lexersMap.count(lexer->GetName()) == 0) { m_lexersMap[lexer->GetName()] = Vec_t(); }
         Vec_t& v = m_lexersMap[lexer->GetName()];
         Vec_t::iterator iter = std::find_if(
             v.begin(), v.end(), [&](LexerConf::Ptr_t l) { return l->GetThemeName() == lexer->GetThemeName(); });
@@ -961,9 +975,7 @@ bool ColoursAndFontsManager::ImportLexersFile(const wxFileName& inputFile, bool
             v.push_back(lexer);
         } else {
             // We dont have this theme, add it
-            if(iter == v.end()) {
-                v.push_back(lexer);
-            }
+            if(iter == v.end()) { v.push_back(lexer); }
         }
     });
 
@@ -987,3 +999,42 @@ wxArrayString ColoursAndFontsManager::GetAllThemes() const
     std::for_each(themes.begin(), themes.end(), [&](const wxString& name) { arr.push_back(name); });
     return arr;
 }
+
+void ColoursAndFontsManager::OnAdjustTheme(clCommandEvent& event)
+{
+    event.Skip();
+    if(event.GetInt() != XRCID("adjust-current-theme")) { return; }
+    event.Skip(false);
+
+    LexerConf::Ptr_t lexer = ColoursAndFontsManager::Get().GetLexer("text");
+    if(!lexer) { return; }
+
+    wxColour bgColour = GetBackgroundColourFromLexer(lexer);
+    if(!bgColour.IsOk()) { return; }
+
+    // Adjust the colours
+    // Save the base colour changes
+    clConfig::Get().Write("BaseColour", bgColour);
+    clConfig::Get().Write("UseCustomBaseColour", true);
+
+    // Notify this change
+    clCommandEvent evt(wxEVT_CMD_COLOURS_FONTS_UPDATED);
+    EventNotifier::Get()->AddPendingEvent(evt);
+
+    clGetManager()->DisplayMessage(_("A CodeLite restart is needed. Would you like to restart it now?"),
+                                   wxICON_QUESTION, { { XRCID("restart-codelite"), _("Yes") }, { wxID_NO, _("No") } });
+}
+
+wxColour ColoursAndFontsManager::GetBackgroundColourFromLexer(LexerConf::Ptr_t lexer) const
+{
+    if(!lexer) { return wxNullColour; }
+    wxColour bgColour;
+    if(lexer->IsDark()) {
+        bgColour = lexer->GetProperty(0).GetBgColour();
+        bgColour = bgColour.ChangeLightness(105);
+    } else {
+        bgColour = lexer->GetProperty(0).GetBgColour();
+        bgColour = bgColour.ChangeLightness(95);
+    }
+    return bgColour;
+}
