@@ -2,7 +2,6 @@
 #define CLTABRENDERERCLASSIC_H
 
 #include "clTabRenderer.h"
-#if !USE_AUI_NOTEBOOK
 class WXDLLIMPEXP_SDK clTabRendererClassic : public clTabRenderer
 {
 public:
@@ -13,9 +12,11 @@ public:
     clTabRendererClassic();
     virtual ~clTabRendererClassic();
     void Draw(wxWindow* parent, wxDC& dc, wxDC& fontDC, const clTabInfo& tabInfo, const clTabColours& colours,
-              size_t style);
+              size_t style, eButtonState buttonState);
     void DrawBottomRect(wxWindow* parent, clTabInfo::Ptr_t tabInfo, const wxRect& clientRect, wxDC& dc,
                         const clTabColours& colours, size_t style);
+    void DrawBackground(wxWindow* parent, wxDC& dc, const wxRect& rect, const clTabColours& colours, size_t style);
+    void FinaliseBackground(wxWindow* parent, wxDC& dc, const wxRect& clientRect, const clTabColours& colours,
+                            size_t style);
 };
-#endif
 #endif // CLTABRENDERERCLASSIC_H
