@@ -10,7 +10,7 @@ public:
     virtual ~clTabRendererCurved();
 
     void Draw(wxWindow* parent, wxDC& dc, wxDC& fontDC, const clTabInfo& tabInfo, const clTabColours& colours,
-              size_t style);
+              size_t style, eButtonState buttonState);
     void DrawBottomRect(wxWindow* parent, clTabInfo::Ptr_t activeTab, const wxRect& clientRect, wxDC& dc,
                         const clTabColours& colours, size_t style);
 };
