@@ -22,17 +22,18 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
-#include <wx/utils.h>
-#include <wx/settings.h>
+#include "bookmark_manager.h"
+#include "drawingutils.h"
+#include "editor_config.h"
 #include "globals.h"
 #include "lexer_configuration.h"
-#include "xmlutils.h"
 #include "macros.h"
 #include "wx_xml_compatibility.h"
-#include "drawingutils.h"
+#include "xmlutils.h"
 #include <algorithm>
-#include "editor_config.h"
-#include "bookmark_manager.h"
+#include <wx/settings.h>
+#include <wx/utils.h>
+#include "cl_config.h"
 
 #ifdef __WXMSW__
 #define DEFAULT_FACE_NAME "Consolas"
@@ -66,8 +67,7 @@ void LexerConf::FromXml(wxXmlNode* element)
         m_lexerId = XmlUtils::ReadLong(element, wxT("Id"), 0);
         m_themeName = XmlUtils::ReadString(element, "Theme", "Default");
         EnableFlag(kIsActive, XmlUtils::ReadBool(element, "IsActive", false));
-        EnableFlag(kUseCustomTextSelectionFgColour,
-                   XmlUtils::ReadBool(element, "UseCustomTextSelFgColour", false));
+        EnableFlag(kUseCustomTextSelectionFgColour, XmlUtils::ReadBool(element, "UseCustomTextSelFgColour", false));
         EnableFlag(kStyleInPP,
                    element->GetPropVal(wxT("StylingWithinPreProcessor"), wxT("yes")) == wxT("yes") ? true : false);
 
@@ -92,9 +92,7 @@ void LexerConf::FromXml(wxXmlNode* element)
             m_extension = node->GetNodeContent();
             // Make sure that CMake includes the CMakeLists.txt file
             if(m_lexerId == wxSTC_LEX_CMAKE) {
-                if(!m_extension.Contains("CMakeLists.txt")) {
-                    m_extension = "*.cmake;*.CMAKE;*CMakeLists.txt";
-                }
+                if(!m_extension.Contains("CMakeLists.txt")) { m_extension = "*.cmake;*.CMAKE;*CMakeLists.txt"; }
             }
         }
 
@@ -132,23 +130,11 @@ void LexerConf::FromXml(wxXmlNode* element)
 
                     // Mainly for upgrade purposes: check if already read
                     // the StringRaw propery
-                    if(isCxxLexer && !hasRawString && propId == wxSTC_C_STRINGRAW) {
-                        hasRawString = true;
-                    }
-                    StyleProperty property = StyleProperty(propId,
-                                                           colour,
-                                                           bgcolour,
-                                                           fontSize,
-                                                           Name,
-                                                           face,
-                                                           StringTolBool(bold),
-                                                           StringTolBool(italic),
-                                                           StringTolBool(underline),
-                                                           StringTolBool(eolFill),
-                                                           alpha);
-                    if(isCxxLexer && propId == wxSTC_C_STRING) {
-                        stringProp = property;
-                    }
+                    if(isCxxLexer && !hasRawString && propId == wxSTC_C_STRINGRAW) { hasRawString = true; }
+                    StyleProperty property =
+                        StyleProperty(propId, colour, bgcolour, fontSize, Name, face, StringTolBool(bold),
+                                      StringTolBool(italic), StringTolBool(underline), StringTolBool(eolFill), alpha);
+                    if(isCxxLexer && propId == wxSTC_C_STRING) { stringProp = property; }
                     m_properties.insert(std::make_pair(propId, property));
                 }
                 prop = prop->GetNext();
@@ -266,19 +252,10 @@ wxFont LexerConf::GetFontForSyle(int styleId) const
 
 static wxColor GetInactiveColor(const StyleProperty& defaultStyle)
 {
-    wxColor inactiveColor;
-    wxColour fgColour = defaultStyle.GetFgColour();
-    if(DrawingUtils::IsDark(defaultStyle.GetBgColour())) {
-        // a dark theme
-        if(DrawingUtils::IsDark(fgColour)) {
-            // this style is using a dark colour, this means that making it darker will not look good - replace it
-            fgColour = wxColour(defaultStyle.GetBgColour()).ChangeLightness(140);
-        }
-        inactiveColor = fgColour.ChangeLightness(30);
-    } else {
-        inactiveColor = wxColour(defaultStyle.GetFgColour()).MakeDisabled();
-    }
-    return inactiveColor;
+    wxColour fg = defaultStyle.GetFgColour();
+    wxColour bg = defaultStyle.GetBgColour();
+    bool is_dark = DrawingUtils::IsDark(bg);
+    return fg.ChangeLightness(is_dark ? 50 : 150);
 }
 
 #define CL_LINE_MODIFIED_STYLE 200
@@ -288,17 +265,22 @@ void LexerConf::Apply(wxStyledTextCtrl* ctrl, bool applyKeywords)
 {
     ctrl->SetLexer(GetLexerId());
     ctrl->StyleClearAll();
-    
+
 #ifndef __WXMSW__
     ctrl->SetStyleBits(ctrl->GetStyleBitsNeeded());
 #endif
 
 #if defined(__WXMSW__)
-    ctrl->SetTechnology(wxSTC_TECHNOLOGY_DIRECTWRITE);
-    //ctrl->SetDoubleBuffered(true);
-    //ctrl->SetFontQuality(wxSTC_EFF_QUALITY_ANTIALIASED);
+    bool useDirect2D = clConfig::Get().Read("Editor/UseDirect2D", true);
+    ctrl->SetTechnology(useDirect2D ? wxSTC_TECHNOLOGY_DIRECTWRITE : wxSTC_TECHNOLOGY_DEFAULT);
+#elif defined(__WXGTK__)
+    // ctrl->SetTechnology(wxSTC_TECHNOLOGY_DIRECTWRITE);
+    // ctrl->SetDoubleBuffered(false);
+#if wxCHECK_VERSION(3, 1, 1)
+    // ctrl->SetFontQuality(wxSTC_EFF_QUALITY_ANTIALIASED);
 #endif
-    
+#endif
+
     OptionsConfigPtr options = EditorConfigST::Get()->GetOptions();
     bool tooltip(false);
 
@@ -308,7 +290,7 @@ void LexerConf::Apply(wxStyledTextCtrl* ctrl, bool applyKeywords)
     // turn off PP tracking/updating by default
     ctrl->SetProperty(wxT("lexer.cpp.track.preprocessor"), wxT("0"));
     ctrl->SetProperty(wxT("lexer.cpp.update.preprocessor"), wxT("0"));
-    
+
     if(GetName() == "scss") {
         // Enable SCSS property (will tell the lexer to search for variables)
         ctrl->SetProperty("lexer.css.scss.language", "1");
@@ -318,7 +300,7 @@ void LexerConf::Apply(wxStyledTextCtrl* ctrl, bool applyKeywords)
     wxFont defaultFont;
     bool foundDefaultStyle = false;
     int nDefaultFontSize = DEFAULT_FONT_SIZE;
-    
+
     StyleProperty defaultStyle;
     StyleProperty::Map_t::const_iterator iter = styles.begin();
     for(; iter != styles.end(); ++iter) {
@@ -326,15 +308,10 @@ void LexerConf::Apply(wxStyledTextCtrl* ctrl, bool applyKeywords)
         if(prop.GetId() == 0) {
             defaultStyle = prop;
             wxString fontFace = prop.GetFaceName().IsEmpty() ? DEFAULT_FACE_NAME : prop.GetFaceName();
-            if(!prop.GetFaceName().IsEmpty()) {
-                nDefaultFontSize = prop.GetFontSize();
-            }
-            defaultFont = wxFont(nDefaultFontSize,
-                                 wxFONTFAMILY_TELETYPE,
-                                 prop.GetItalic() ? wxFONTSTYLE_ITALIC : wxFONTSTYLE_NORMAL,
-                                 prop.IsBold() ? wxFONTWEIGHT_BOLD : wxFONTWEIGHT_NORMAL,
-                                 prop.GetUnderlined(),
-                                 fontFace);
+            if(!prop.GetFaceName().IsEmpty()) { nDefaultFontSize = prop.GetFontSize(); }
+            defaultFont = wxFont(
+                nDefaultFontSize, wxFONTFAMILY_TELETYPE, prop.GetItalic() ? wxFONTSTYLE_ITALIC : wxFONTSTYLE_NORMAL,
+                prop.IsBold() ? wxFONTWEIGHT_BOLD : wxFONTWEIGHT_NORMAL, prop.GetUnderlined(), fontFace);
             foundDefaultStyle = true;
             break;
         }
@@ -369,9 +346,7 @@ void LexerConf::Apply(wxStyledTextCtrl* ctrl, bool applyKeywords)
         case WHITE_SPACE_ATTR_ID: {
             // whitespace colour. We dont allow changing the background colour, only the foreground colour
             wxColour whitespaceColour = sp.GetFgColour();
-            if(whitespaceColour.IsOk()) {
-                ctrl->SetWhitespaceForeground(true, whitespaceColour);
-            }
+            if(whitespaceColour.IsOk()) { ctrl->SetWhitespaceForeground(true, whitespaceColour); }
             break;
         }
         case FOLD_MARGIN_ATTR_ID:
@@ -381,9 +356,7 @@ void LexerConf::Apply(wxStyledTextCtrl* ctrl, bool applyKeywords)
             break;
         case SEL_TEXT_ATTR_ID: {
             // selection colour
-            if(wxColour(sp.GetBgColour()).IsOk()) {
-                ctrl->SetSelBackground(true, sp.GetBgColour());
-            }
+            if(wxColour(sp.GetBgColour()).IsOk()) { ctrl->SetSelBackground(true, sp.GetBgColour()); }
             if(IsUseCustomTextSelectionFgColour() && wxColour(sp.GetFgColour()).IsOk()) {
                 ctrl->SetSelForeground(true, sp.GetFgColour());
             } else {
@@ -396,9 +369,7 @@ void LexerConf::Apply(wxStyledTextCtrl* ctrl, bool applyKeywords)
         case CARET_ATTR_ID: {
             // caret colour
             wxColour caretColour = sp.GetFgColour();
-            if(!caretColour.IsOk()) {
-                caretColour = *wxBLACK;
-            }
+            if(!caretColour.IsOk()) { caretColour = *wxBLACK; }
             ctrl->SetCaretForeground(caretColour);
             break;
         }
@@ -522,17 +493,15 @@ void LexerConf::Apply(wxStyledTextCtrl* ctrl, bool applyKeywords)
     // Define the styles for the editing margin
     ctrl->StyleSetBackground(CL_LINE_SAVED_STYLE, wxColour(wxT("FOREST GREEN")));
     ctrl->StyleSetBackground(CL_LINE_MODIFIED_STYLE, wxColour(wxT("ORANGE")));
-    
+
     // Indentation
     ctrl->SetUseTabs(options->GetIndentUsesTabs());
     ctrl->SetTabWidth(options->GetTabWidth());
     ctrl->SetIndent(options->GetIndentWidth());
-    
+
     // Overide TAB vs Space settings incase the file is a makefile
     // It is not an option for Makefile to use SPACES
-    if(GetName().Lower() == "makefile") {
-        ctrl->SetUseTabs(true);
-    }
+    if(GetName().Lower() == "makefile") { ctrl->SetUseTabs(true); }
 }
 
 const StyleProperty& LexerConf::GetProperty(int propertyId) const
@@ -560,31 +529,25 @@ StyleProperty& LexerConf::GetProperty(int propertyId)
 bool LexerConf::IsDark() const
 {
     const StyleProperty& prop = GetProperty(0);
-    if(prop.IsNull()) {
-        return false;
-    }
+    if(prop.IsNull()) { return false; }
     return DrawingUtils::IsDark(prop.GetBgColour());
 }
 
 void LexerConf::SetDefaultFgColour(const wxColour& colour)
 {
     StyleProperty& style = GetProperty(0);
-    if(!style.IsNull()) {
-        style.SetFgColour(colour.GetAsString(wxC2S_HTML_SYNTAX));
-    }
+    if(!style.IsNull()) { style.SetFgColour(colour.GetAsString(wxC2S_HTML_SYNTAX)); }
 }
 
 void LexerConf::SetLineNumbersFgColour(const wxColour& colour)
 {
     StyleProperty& style = GetProperty(LINE_NUMBERS_ATTR_ID);
-    if(!style.IsNull()) {
-        style.SetFgColour(colour.GetAsString(wxC2S_HTML_SYNTAX));
-    }
+    if(!style.IsNull()) { style.SetFgColour(colour.GetAsString(wxC2S_HTML_SYNTAX)); }
 }
 
-JSONElement LexerConf::ToJSON() const
+JSONItem LexerConf::ToJSON(bool forExport) const
 {
-    JSONElement json = JSONElement::createObject(GetName());
+    JSONItem json = JSONItem::createObject(GetName());
     json.addProperty("Name", GetName());
     json.addProperty("Theme", GetThemeName());
     json.addProperty("Flags", m_flags);
@@ -596,17 +559,17 @@ JSONElement LexerConf::ToJSON() const
     json.addProperty("KeyWords4", GetKeyWords(4));
     json.addProperty("Extensions", GetFileSpec());
 
-    JSONElement properties = JSONElement::createArray("Properties");
+    JSONItem properties = JSONItem::createArray("Properties");
     json.append(properties);
 
     StyleProperty::Map_t::const_iterator iter = m_properties.begin();
     for(; iter != m_properties.end(); ++iter) {
-        properties.arrayAppend(iter->second.ToJSON());
+        properties.arrayAppend(iter->second.ToJSON(forExport));
     }
     return json;
 }
 
-void LexerConf::FromJSON(const JSONElement& json)
+void LexerConf::FromJSON(const JSONItem& json)
 {
     m_name = json.namedObject("Name").toString();
     m_lexerId = json.namedObject("Id").toInt();
@@ -626,7 +589,7 @@ void LexerConf::FromJSON(const JSONElement& json)
     SetFileSpec(json.namedObject("Extensions").toString());
 
     m_properties.clear();
-    JSONElement properties = json.namedObject("Properties");
+    JSONItem properties = json.namedObject("Properties");
     int arrSize = properties.arraySize();
     for(int i = 0; i < arrSize; ++i) {
         // Construct a style property
