@@ -73,6 +73,10 @@ public:
 
     enum {
         Opt2_MouseScrollSwitchTabs = (1 << 0),
+        Opt2_SortTabsDropdownAlphabetically = (1 << 1),
+        Opt2_PlaceNavBarAtTheTop = (1 << 2),
+        Opt2_DisableCtrlTabForTabSwitching = (1 << 3),
+        Opt2_SortNavBarDropdown = (1 << 4),
     };
 
     enum { nbTabHt_Tiny = 1, nbTabHt_Short, nbTabHt_Medium, nbTabHt_Tall };
@@ -90,6 +94,8 @@ protected:
     bool m_highlightCaretLine;
     bool m_clearHighlitWordsOnFind;
     bool m_displayLineNumbers;
+    bool m_relativeLineNumbers = false;
+    bool m_highlightCurLineNumber = true;
     bool m_showIndentationGuidelines;
     wxColour m_caretLineColour;
     bool m_indentUsesTabs;
@@ -109,6 +115,7 @@ protected:
     bool m_autoAdjustHScrollBarWidth;
     int m_caretWidth;
     int m_caretBlinkPeriod;
+    bool m_copyLineEmptySelection;
     wxString m_programConsoleCommand;
     wxString m_eolMode;
     bool m_hideChangeMarkerMargin;
@@ -135,6 +142,7 @@ protected:
     bool m_disableSemicolonShift;
     int m_caretLineAlpha;
     bool m_dontAutoFoldResults;
+    bool m_dontOverrideSearchStringWithSelection;
     bool m_showDebugOnRun;
     bool m_caretUseCamelCase;
     bool m_dontTrimCaretLine;
@@ -151,7 +159,8 @@ protected:
     wxDirection m_outputTabsDirection;    // Up/Down
     bool m_indentedComments;
     int m_nbTabHeight; // Should notebook tabs be too tall, too short or...
-    
+    wxString m_webSearchPrefix;
+
 public:
     // Helpers
     void EnableOption(size_t flag, bool b)
@@ -202,6 +211,14 @@ public:
     bool IsTabHasXButton() const { return !HasOption(Opt_TabNoXButton); }
     bool IsMouseScrollSwitchTabs() const { return HasOption2(Opt2_MouseScrollSwitchTabs); }
     void SetMouseScrollSwitchTabs(bool b) { EnableOption2(Opt2_MouseScrollSwitchTabs, b); }
+    bool IsSortTabsDropdownAlphabetically() const { return HasOption2(Opt2_SortTabsDropdownAlphabetically); }
+    void SetSortTabsDropdownAlphabetically(bool b) { EnableOption2(Opt2_SortTabsDropdownAlphabetically, b); }
+    bool IsNavBarTop() const { return HasOption2(Opt2_PlaceNavBarAtTheTop); }
+    void SetNavBarTop(bool b) { EnableOption2(Opt2_PlaceNavBarAtTheTop, b); }
+    bool IsCtrlTabEnabled() const { return !HasOption2(Opt2_DisableCtrlTabForTabSwitching); }
+    void SetCtrlTabEnabled(bool b) { EnableOption2(Opt2_DisableCtrlTabForTabSwitching, !b); }
+    bool IsSortNavBarDropdown() const { return HasOption2(Opt2_SortNavBarDropdown); }
+    void SetSortNavBarDropdown(bool b) { EnableOption2(Opt2_SortNavBarDropdown, b); }
 
     void SetOptions(size_t options) { this->m_options = options; }
     size_t GetOptions() const { return m_options; }
@@ -219,6 +236,8 @@ public:
     bool GetCaretUseCamelCase() const { return m_caretUseCamelCase; }
     void SetDontAutoFoldResults(bool dontAutoFoldResults) { this->m_dontAutoFoldResults = dontAutoFoldResults; }
     bool GetDontAutoFoldResults() const { return m_dontAutoFoldResults; }
+    void SetDontOverrideSearchStringWithSelection(bool dontOverrideSearchStringWithSelection) { m_dontOverrideSearchStringWithSelection = dontOverrideSearchStringWithSelection; }
+    bool GetDontOverrideSearchStringWithSelection() const { return m_dontOverrideSearchStringWithSelection; }
     void SetShowDebugOnRun(bool showDebugOnRun) { this->m_showDebugOnRun = showDebugOnRun; }
     bool GetShowDebugOnRun() const { return m_showDebugOnRun; }
     bool GetDisableSemicolonShift() const { return m_disableSemicolonShift; }
@@ -332,6 +351,8 @@ public:
 
     bool GetHighlightCaretLine() const { return m_highlightCaretLine; }
     bool GetDisplayLineNumbers() const { return m_displayLineNumbers; }
+    bool GetRelativeLineNumbers() const { return m_relativeLineNumbers; }
+    bool GetHighlightCurrentLineNumber() const { return m_highlightCurLineNumber; }
     bool GetShowIndentationGuidelines() const { return m_showIndentationGuidelines; }
     wxColour GetCaretLineColour() const { return m_caretLineColour; }
 
@@ -352,6 +373,8 @@ public:
 
     void SetHighlightCaretLine(bool b) { m_highlightCaretLine = b; }
     void SetDisplayLineNumbers(bool b) { m_displayLineNumbers = b; }
+    void SetRelativeLineNumbers(bool b) { m_relativeLineNumbers = b; }
+    void SetHighlightCurrentLineNumber(bool b) { m_highlightCurLineNumber = b; }
     void SetShowIndentationGuidelines(bool b) { m_showIndentationGuidelines = b; }
     void SetCaretLineColour(wxColour c) { m_caretLineColour = c; }
 
@@ -418,6 +441,9 @@ public:
     const int& GetCaretBlinkPeriod() const { return m_caretBlinkPeriod; }
     const int& GetCaretWidth() const { return m_caretWidth; }
 
+    void SetCopyLineEmptySelection(const bool copyLineEmptySelection) { m_copyLineEmptySelection = copyLineEmptySelection; }
+    bool GetCopyLineEmptySelection() const { return m_copyLineEmptySelection; }
+
     void SetProgramConsoleCommand(const wxString& programConsoleCommand)
     {
         this->m_programConsoleCommand = programConsoleCommand;
@@ -453,6 +479,9 @@ public:
     bool IsMouseZoomEnabled() const { return !HasOption(Opt_DisableMouseCtrlZoom); }
     void SetMouseZoomEnabled(bool b) { EnableOption(Opt_DisableMouseCtrlZoom, !b); }
 
+    const wxString& GetWebSearchPrefix() const { return m_webSearchPrefix; }
+    void SetWebSearchPrefix(const wxString& webSearchPrefix) { this->m_webSearchPrefix = webSearchPrefix; }
+
     void UpdateFromEditorConfig(const clEditorConfigSection& section);
 
     /**
