@@ -42,25 +42,30 @@
 #include <wx/msgdlg.h>
 #include <wx/thread.h>
 #include <wx/tokenzr.h>
+#include "file_logger.h"
+#include "macromanager.h"
+#include "build_settings_config.h"
+#include "localworkspace.h"
+#include "compiler_command_line_parser.h"
+#include "fileutils.h"
 
 clCxxWorkspace::clCxxWorkspace()
     : m_saveOnExit(true)
 {
     SetWorkspaceType(_("C++"));
+    m_localWorkspace = new LocalWorkspace();
 }
 
 clCxxWorkspace::~clCxxWorkspace()
 {
-    if(m_saveOnExit && m_doc.IsOk()) {
-        SaveXmlFile();
-    }
+    if(m_saveOnExit && m_doc.IsOk()) { SaveXmlFile(); }
+    delete m_localWorkspace;
+    m_localWorkspace = nullptr;
 }
 
 wxString clCxxWorkspace::GetName() const
 {
-    if(m_doc.IsOk()) {
-        return XmlUtils::ReadString(m_doc.GetRoot(), wxT("Name"));
-    }
+    if(m_doc.IsOk()) { return XmlUtils::ReadString(m_doc.GetRoot(), wxT("Name")); }
     return wxEmptyString;
 }
 
@@ -83,14 +88,10 @@ bool clCxxWorkspace::OpenReadOnly(const wxString& fileName, wxString& errMsg)
 {
     m_buildMatrix.Reset(NULL);
     wxFileName workSpaceFile(fileName);
-    if(!workSpaceFile.FileExists()) {
-        return false;
-    }
+    if(!workSpaceFile.FileExists()) { return false; }
     m_fileName = workSpaceFile;
     m_doc.Load(m_fileName.GetFullPath());
-    if(!m_doc.IsOk()) {
-        return false;
-    }
+    if(!m_doc.IsOk()) { return false; }
 
     m_saveOnExit = false;
 
@@ -110,9 +111,7 @@ bool clCxxWorkspace::OpenReadOnly(const wxString& fileName, wxString& errMsg)
 
 bool clCxxWorkspace::OpenWorkspace(const wxString& fileName, wxString& errMsg)
 {
-    if(!DoLoadWorkspace(fileName, errMsg)) {
-        return false;
-    }
+    if(!DoLoadWorkspace(fileName, errMsg)) { return false; }
 
     // Notify about active project changed
     ProjectPtr activeProject = GetActiveProject();
@@ -201,7 +200,7 @@ bool clCxxWorkspace::CreateWorkspace(const wxString& name, const wxString& path,
     m_doc.GetRoot()->AddProperty(wxT("Database"), dbFileName.GetFullPath(wxPATH_UNIX));
 
     m_doc.GetRoot()->DeleteAttribute(wxT("SWTLW"));
-    if(LocalWorkspaceST::Get()->GetParserFlags() & LocalWorkspace::EnableSWTLW) {
+    if(GetLocalWorkspace()->GetParserFlags() & LocalWorkspace::EnableSWTLW) {
         m_doc.GetRoot()->AddProperty(wxT("SWTLW"), "Yes");
     }
 
@@ -345,14 +344,10 @@ bool clCxxWorkspace::CreateProject(const wxString& name, const wxString& path, c
     wxXmlNode* parentNode = DoCreateWorkspaceFolder(workspaceFolder);
     parentNode->AddChild(node);
 
-    if(m_projects.size() == 1) {
-        SetActiveProject(name);
-    }
+    if(m_projects.size() == 1) { SetActiveProject(name); }
 
     SaveXmlFile();
-    if(addToBuildMatrix) {
-        AddProjectToBuildMatrix(proj);
-    }
+    if(addToBuildMatrix) { AddProjectToBuildMatrix(proj); }
     return true;
 }
 
@@ -386,9 +381,7 @@ bool clCxxWorkspace::AddProject(const wxString& path, // fullpath
     wxXmlNode* parentNode = DoCreateWorkspaceFolder(workspaceFolder);
     parentNode->AddChild(node);
 
-    if(m_projects.size() == 1) {
-        SetActiveProject(proj->GetName());
-    }
+    if(m_projects.size() == 1) { SetActiveProject(proj->GetName()); }
 
     SaveXmlFile();
     AddProjectToBuildMatrix(proj);
@@ -413,6 +406,7 @@ ProjectPtr clCxxWorkspace::FindProjectByName(const wxString& projName, wxString&
 
 void clCxxWorkspace::GetProjectList(wxArrayString& list) const
 {
+    list.reserve(m_projects.size());
     ProjectMap_t::const_iterator iter = m_projects.begin();
     for(; iter != m_projects.end(); iter++) {
         wxString name;
@@ -476,9 +470,7 @@ bool clCxxWorkspace::AddProject(const wxString& path, wxString& errMsg)
 
 ProjectPtr clCxxWorkspace::DoAddProject(ProjectPtr proj)
 {
-    if(!proj) {
-        return NULL;
-    }
+    if(!proj) { return NULL; }
 
     m_projects.insert(std::make_pair(proj->GetName(), proj));
     proj->AssociateToWorkspace(this);
@@ -492,9 +484,7 @@ ProjectPtr clCxxWorkspace::DoAddProject(const wxString& path, const wxString& pr
 
     // Convert the path to absolute path
     wxFileName projectFile(path);
-    if(projectFile.IsRelative()) {
-        projectFile.MakeAbsolute(m_fileName.GetPath());
-    }
+    if(projectFile.IsRelative()) { projectFile.MakeAbsolute(m_fileName.GetPath()); }
 
     if(!proj->Load(projectFile.GetFullPath())) {
         errMsg = wxT("Corrupted project file '");
@@ -512,9 +502,7 @@ ProjectPtr clCxxWorkspace::DoAddProject(const wxString& path, const wxString& pr
 bool clCxxWorkspace::RemoveProject(const wxString& name, wxString& errMsg, const wxString& workspaceFolder)
 {
     ProjectPtr proj = FindProjectByName(name, errMsg);
-    if(!proj) {
-        return false;
-    }
+    if(!proj) { return false; }
 
     // remove the associated build configuration with this
     // project
@@ -522,9 +510,7 @@ bool clCxxWorkspace::RemoveProject(const wxString& name, wxString& errMsg, const
 
     // remove the project from the internal map
     ProjectMap_t::iterator iter = m_projects.find(proj->GetName());
-    if(iter != m_projects.end()) {
-        m_projects.erase(iter);
-    }
+    if(iter != m_projects.end()) { m_projects.erase(iter); }
 
     // update the xml file
     // Incase we got a workspace folder, the project node will exists under the
@@ -532,9 +518,7 @@ bool clCxxWorkspace::RemoveProject(const wxString& name, wxString& errMsg, const
     wxXmlNode* root = m_doc.GetRoot();
     if(!workspaceFolder.IsEmpty()) {
         wxXmlNode* node = DoGetWorkspaceFolderXmlNode(workspaceFolder);
-        if(node) {
-            root = node;
-        }
+        if(node) { root = node; }
     }
 
     wxXmlNode* child = root->GetChildren();
@@ -577,9 +561,7 @@ bool clCxxWorkspace::RemoveProject(const wxString& name, wxString& errMsg, const
 
                 wxArrayString deps = p->GetDependencies(configs.Item(i));
                 int where = deps.Index(name);
-                if(where != wxNOT_FOUND) {
-                    deps.RemoveAt((size_t)where);
-                }
+                if(where != wxNOT_FOUND) { deps.RemoveAt((size_t)where); }
 
                 // update the configuration
                 p->SetDependencies(deps, configs.Item(i));
@@ -591,9 +573,7 @@ bool clCxxWorkspace::RemoveProject(const wxString& name, wxString& errMsg, const
 
 wxString clCxxWorkspace::GetActiveProjectName() const
 {
-    if(!m_doc.IsOk()) {
-        return wxEmptyString;
-    }
+    if(!m_doc.IsOk()) { return wxEmptyString; }
 
     std::list<wxXmlNode*> xmls = DoGetProjectsXmlNodes();
     std::list<wxXmlNode*>::iterator iter = std::find_if(xmls.begin(), xmls.end(), [&](wxXmlNode* node) {
@@ -662,11 +642,9 @@ bool clCxxWorkspace::SaveXmlFile()
     // We first remove the Save Workspace To Local Workspace (SWTLW) attribute
     // and then check the current state in the Code Completion tab. Then
     // we read new path values from the LW and set the appropiate attribute value.
-    if(m_doc.GetRoot()->GetAttribute(wxT("SWTLW")) != wxEmptyString) {
-        m_doc.GetRoot()->DeleteAttribute(wxT("SWTLW"));
-    }
+    if(m_doc.GetRoot()->GetAttribute(wxT("SWTLW")) != wxEmptyString) { m_doc.GetRoot()->DeleteAttribute(wxT("SWTLW")); }
 
-    if(LocalWorkspaceST::Get()->GetParserFlags() & LocalWorkspace::EnableSWTLW) {
+    if(GetLocalWorkspace()->GetParserFlags() & LocalWorkspace::EnableSWTLW) {
         m_doc.GetRoot()->AddProperty(wxT("SWTLW"), "Yes");
         SyncFromLocalWorkspaceSTParserPaths();
         SyncFromLocalWorkspaceSTParserMacros();
@@ -696,22 +674,18 @@ void clCxxWorkspace::SyncToLocalWorkspaceSTParserPaths()
             if(child->GetName() == wxT("Exclude")) {
                 wxString path = child->GetPropVal(wxT("Path"), wxT(""));
                 path.Trim().Trim(false);
-                if(path.IsEmpty() == false) {
-                    excludePaths.Add(path);
-                }
+                if(path.IsEmpty() == false) { excludePaths.Add(path); }
             }
 
             else if(child->GetName() == wxT("Include")) {
                 wxString path = child->GetPropVal(wxT("Path"), wxT(""));
                 path.Trim().Trim(false);
-                if(path.IsEmpty() == false) {
-                    inclduePaths.Add(path);
-                }
+                if(path.IsEmpty() == false) { inclduePaths.Add(path); }
             }
 
             child = child->GetNext();
         }
-        LocalWorkspaceST::Get()->SetParserPaths(inclduePaths, excludePaths);
+        GetLocalWorkspace()->SetParserPaths(inclduePaths, excludePaths);
     }
 }
 
@@ -731,7 +705,7 @@ void clCxxWorkspace::SyncFromLocalWorkspaceSTParserPaths()
     //
     wxArrayString inclduePaths;
     wxArrayString excludePaths;
-    LocalWorkspaceST::Get()->GetParserPaths(inclduePaths, excludePaths);
+    GetLocalWorkspace()->GetParserPaths(inclduePaths, excludePaths);
 
     workspaceInclPaths = new wxXmlNode(m_doc.GetRoot(), wxXML_ELEMENT_NODE, wxT("WorkspaceParserPaths"));
     for(size_t i = 0; i < inclduePaths.GetCount(); i++) {
@@ -752,7 +726,7 @@ void clCxxWorkspace::SyncToLocalWorkspaceSTParserMacros()
     if(workspaceMacros) {
         macros = workspaceMacros->GetNodeContent();
         macros.Trim().Trim(false);
-        LocalWorkspaceST::Get()->SetParserMacros(macros);
+        GetLocalWorkspace()->SetParserMacros(macros);
     }
 }
 
@@ -771,7 +745,7 @@ void clCxxWorkspace::SyncFromLocalWorkspaceSTParserMacros()
     // Get workspace parse macros from local workspace file.
     //
     wxString macros;
-    LocalWorkspaceST::Get()->GetParserMacros(macros);
+    GetLocalWorkspace()->GetParserMacros(macros);
     workspaceMacros = new wxXmlNode(m_doc.GetRoot(), wxXML_ELEMENT_NODE, wxT("WorkspaceParserMacros"));
     if(!macros.IsEmpty()) {
         wxXmlNode* contentNode = new wxXmlNode(wxXML_CDATA_SECTION_NODE, wxEmptyString, macros);
@@ -844,18 +818,14 @@ bool clCxxWorkspace::RemoveFile(const wxString& vdFullPath, const wxString& file
     }
 
     bool result = proj->RemoveFile(fileName, fixedPath);
-    if(!result) {
-        errMsg = _("File removal failed");
-    }
+    if(!result) { errMsg = _("File removal failed"); }
     return result;
 }
 
 BuildConfigPtr clCxxWorkspace::GetProjBuildConf(const wxString& projectName, const wxString& confName) const
 {
     BuildMatrixPtr matrix = GetBuildMatrix();
-    if(!matrix) {
-        return NULL;
-    }
+    if(!matrix) { return NULL; }
 
     wxString projConf(confName);
 
@@ -869,9 +839,7 @@ BuildConfigPtr clCxxWorkspace::GetProjBuildConf(const wxString& projectName, con
     ProjectPtr proj = FindProjectByName(projectName, errMsg);
     if(proj) {
         ProjectSettingsPtr settings = proj->GetSettings();
-        if(settings) {
-            return settings->GetBuildConfiguration(projConf, true);
-        }
+        if(settings) { return settings->GetBuildConfiguration(projConf, true); }
     }
     return NULL;
 }
@@ -888,9 +856,7 @@ void clCxxWorkspace::ReloadWorkspace()
     mgr->CloseDatabase();
 
     wxString err_msg;
-    if(!OpenWorkspace(m_fileName.GetFullPath(), err_msg)) {
-        wxLogMessage(wxT("Reload workspace: ") + err_msg);
-    }
+    if(!OpenWorkspace(m_fileName.GetFullPath(), err_msg)) { clDEBUG() << "Reload workspace:" << err_msg; }
 }
 
 time_t clCxxWorkspace::GetFileLastModifiedTime() const { return GetFileModificationTime(GetWorkspaceFileName()); }
@@ -899,9 +865,7 @@ time_t clCxxWorkspace::GetFileLastModifiedTime() const { return GetFileModificat
 static clCxxWorkspace* gs_Workspace = NULL;
 void clCxxWorkspaceST::Free()
 {
-    if(gs_Workspace) {
-        delete gs_Workspace;
-    }
+    if(gs_Workspace) { delete gs_Workspace; }
     gs_Workspace = NULL;
 }
 
@@ -932,9 +896,7 @@ wxString clCxxWorkspace::GetEnvironmentVariabels()
     wxString env;
     if(GetSelectedConfig()) {
         env = GetSelectedConfig()->GetEnvironmentVariables();
-        if(!env.IsEmpty()) {
-            return env;
-        }
+        if(!env.IsEmpty()) { return env; }
     }
 
     wxXmlNode* node = XmlUtils::FindFirstByTagName(m_doc.GetRoot(), wxT("Environment"));
@@ -1002,9 +964,7 @@ bool clCxxWorkspace::IsVirtualDirectoryExists(const wxString& vdFullPath)
 
     wxString errMsg;
     ProjectPtr proj = FindProjectByName(projName, errMsg);
-    if(!proj) {
-        return false;
-    }
+    if(!proj) { return false; }
 
     wxXmlNode* vdNode = proj->GetVirtualDir(fixedPath);
     return vdNode != NULL;
@@ -1020,9 +980,7 @@ wxString clCxxWorkspace::GetPrivateFolder() const
         clCommandEvent event(wxEVT_CMD_IS_WORKSPACE_OPEN);
         event.SetAnswer(false);
         EventNotifier::Get()->ProcessEvent(event);
-        if(event.IsAnswer()) {
-            workspacePath = event.GetFileName();
-        }
+        if(event.IsAnswer()) { workspacePath = event.GetFileName(); }
     }
     if(workspacePath.Exists()) {
         // append the .codelite folder
@@ -1038,30 +996,90 @@ wxString clCxxWorkspace::GetPrivateFolder() const
 
 wxFileName clCxxWorkspace::GetTagsFileName() const
 {
-    if(!IsOpen()) {
-        return wxFileName();
-    }
+    if(!IsOpen()) { return wxFileName(); }
 
     wxFileName fn_tags(GetPrivateFolder(), GetWorkspaceFileName().GetFullName());
+    fn_tags.SetName(fn_tags.GetName() + "-" + ::clGetUserName());
     fn_tags.SetExt("tags");
     return fn_tags;
 }
 
-void clCxxWorkspace::CreateCompileCommandsJSON(JSONElement& compile_commands) const
+cJSON* clCxxWorkspace::CreateCompileCommandsJSON() const
 {
-    BuildMatrixPtr matrix = clCxxWorkspaceST::Get()->GetBuildMatrix();
-    if(!matrix) return;
+    // Build the global compiler paths, we will need this later on...
+    wxStringMap_t compilersGlobalPaths;
+    std::unordered_map<wxString, wxArrayString> pathsMap = BuildSettingsConfigST::Get()->GetCompilersGlobalPaths();
+    for(const auto& vt : pathsMap) {
+        wxString compiler_name = vt.first;
+        wxArrayString pathsArr = vt.second;
+        wxString paths;
+        std::for_each(pathsArr.begin(), pathsArr.end(), [&](wxString& path) {
+            path.Trim().Trim(false);
+            if(path.EndsWith("\\")) { path.RemoveLast(); }
+            paths << path << ";";
+        });
+        compilersGlobalPaths.insert({ compiler_name, paths });
+    }
 
-    wxString workspaceSelConf = matrix->GetSelectedConfigurationName();
+    // Check if the active project is using custom build
+    ProjectPtr activeProject = GetActiveProject();
+    if(activeProject) {
+        BuildConfigPtr buildConf = activeProject->GetBuildConfiguration();
+        if(buildConf && buildConf->IsCustomBuild()) {
+            // Using custom build
+            wxString buildWorkingDirectory = MacroManager::Instance()->Expand(
+                buildConf->GetCustomBuildWorkingDir(), NULL, activeProject->GetName(), buildConf->GetName());
+            wxFileName fnWorkingDirectory(buildWorkingDirectory, "compile_commands.json");
+            if(fnWorkingDirectory.FileExists()) {
+
+                JSON root(fnWorkingDirectory);
+                if(root.isOk()) {
+                    JSON newFile(cJSON_Array);
+                    JSONItem newArr = newFile.toElement();
+                    JSONItem arr = root.toElement();
+                    int size = arr.arraySize();
+                    for(int i = 0; i < size; ++i) {
+                        JSONItem file = arr.arrayItem(i);
+                        wxString command = file.namedObject("command").toString();
+                        wxString file_name = file.namedObject("file").toString();
+                        wxString directory = file.namedObject("directory").toString();
+
+                        std::vector<wxString> files;
+                        files.push_back(file_name);
+
+                        wxFileName headerFile(file_name);
+                        headerFile.SetExt("h");
+                        if(headerFile.FileExists()) { files.push_back(headerFile.GetFullPath()); }
+                        
+                        // Create two entries: header & cpp (if the header exists)
+                        for(const wxString& filename : files) {
+                            JSONItem fileItem = JSONItem::createObject();
+                            fileItem.addProperty("file", filename).addProperty("directory", directory);
+
+                            // Fix the build command
+                            CompilerCommandLineParser cclp(command, directory);
+                            cclp.MakeAbsolute(directory);
+                            fileItem.addProperty("command",
+                                                 wxString() << "clang " << cclp.GetCompileLine() << " -c " << filename);
+                            newArr.arrayAppend(fileItem);
+                        }
+                    }
+                    return newFile.release();
+                }
+            }
+        }
+    }
+
+    JSONItem compile_commands = JSONItem::createArray();
     clCxxWorkspace::ProjectMap_t::const_iterator iter = m_projects.begin();
-
     for(; iter != m_projects.end(); ++iter) {
         BuildConfigPtr buildConf = iter->second->GetBuildConfiguration();
         if(buildConf && buildConf->IsProjectEnabled() && !buildConf->IsCustomBuild() &&
            buildConf->IsCompilerRequired()) {
-            iter->second->CreateCompileCommandsJSON(compile_commands);
+            iter->second->CreateCompileCommandsJSON(compile_commands, compilersGlobalPaths);
         }
     }
+    return compile_commands.release();
 }
 
 ProjectPtr clCxxWorkspace::GetActiveProject() const { return GetProject(GetActiveProjectName()); }
@@ -1069,9 +1087,7 @@ ProjectPtr clCxxWorkspace::GetActiveProject() const { return GetProject(GetActiv
 ProjectPtr clCxxWorkspace::GetProject(const wxString& name) const
 {
     clCxxWorkspace::ProjectMap_t::const_iterator iter = m_projects.find(name);
-    if(iter == m_projects.end()) {
-        return NULL;
-    }
+    if(iter == m_projects.end()) { return NULL; }
     return iter->second;
 }
 
@@ -1108,9 +1124,7 @@ void clCxxWorkspace::RenameProject(const wxString& oldname, const wxString& newn
                 while(projectNode) {
                     if(projectNode->GetName() == "Project") {
                         wxString name = projectNode->GetAttribute("Name");
-                        if(name == oldname) {
-                            XmlUtils::UpdateProperty(projectNode, "Name", newname);
-                        }
+                        if(name == oldname) { XmlUtils::UpdateProperty(projectNode, "Name", newname); }
                     }
                     projectNode = projectNode->GetNext();
                 }
@@ -1122,9 +1136,7 @@ void clCxxWorkspace::RenameProject(const wxString& oldname, const wxString& newn
     // Update the list of projects in the workspace
     wxXmlNode* projectNode = XmlUtils::FindFirstByTagName(m_doc.GetRoot(), "Project");
     while(projectNode) {
-        if(projectNode->GetAttribute("Name") == oldname) {
-            XmlUtils::UpdateProperty(projectNode, "Name", newname);
-        }
+        if(projectNode->GetAttribute("Name") == oldname) { XmlUtils::UpdateProperty(projectNode, "Name", newname); }
         projectNode = projectNode->GetNext();
     }
     // Update dependenices for each project
@@ -1158,10 +1170,8 @@ wxString clCxxWorkspace::GetFilesMask() const
     wxString findInFilesMask = "*.c;*.cpp;*.cxx;*.cc;*.h;*.hpp;*.inc;*.mm;*.m;*.xrc;*.ini;*.xml";
     if(IsOpen()) {
         wxString fifMask;
-        LocalWorkspaceST::Get()->GetSearchInFilesMask(fifMask, findInFilesMask);
-        if(fifMask.IsEmpty()) {
-            fifMask = findInFilesMask;
-        }
+        GetLocalWorkspace()->GetSearchInFilesMask(fifMask, findInFilesMask);
+        if(fifMask.IsEmpty()) { fifMask = findInFilesMask; }
     }
     return findInFilesMask;
 }
@@ -1170,9 +1180,7 @@ wxString clCxxWorkspace::GetProjectFromFile(const wxFileName& filename) const
     wxString filenameFP = filename.GetFullPath();
     clCxxWorkspace::ProjectMap_t::const_iterator iter = m_projects.begin();
     for(; iter != m_projects.end(); ++iter) {
-        if(iter->second->GetFiles().count(filenameFP)) {
-            return iter->first;
-        }
+        if(iter->second->GetFiles().count(filenameFP)) { return iter->first; }
     }
     return "";
 }
@@ -1183,9 +1191,7 @@ void clCxxWorkspace::GetProjectFiles(const wxString& projectName, wxArrayString&
     CHECK_PTR_RET(p);
 
     const Project::FilesMap_t& filesMap = p->GetFiles();
-    if(filesMap.empty()) {
-        return;
-    }
+    if(filesMap.empty()) { return; }
     files.Alloc(filesMap.size());
     std::for_each(filesMap.begin(), filesMap.end(),
                   [&](const Project::FilesMap_t::value_type& vt) { files.Add(vt.first); });
@@ -1239,24 +1245,20 @@ void clCxxWorkspace::DoLoadProjectsFromXml(wxXmlNode* parentNode, const wxString
         if(child->GetName() == wxT("Project")) {
             wxString projectPath = child->GetPropVal(wxT("Path"), wxEmptyString);
             wxString errmsg;
-            if(!DoAddProject(projectPath, folder, errmsg)) {
-                removedChildren.push_back(child);
-            }
+            if(!DoAddProject(projectPath, folder, errmsg)) { removedChildren.push_back(child); }
         } else if(child->GetName() == wxT("VirtualDirectory")) {
             // Virtual directory
             wxString currentFolder = folder;
             wxString vdName = child->GetAttribute("Name", wxEmptyString);
-            if(!currentFolder.IsEmpty()) {
-                currentFolder << "/";
-            }
+            if(!currentFolder.IsEmpty()) { currentFolder << "/"; }
             currentFolder << vdName;
             DoLoadProjectsFromXml(child, currentFolder, removedChildren);
         } else if((child->GetName() == wxT("WorkspaceParserPaths")) ||
                   (child->GetName() == wxT("WorkspaceParserMacros"))) {
             wxString swtlw = XmlUtils::ReadString(m_doc.GetRoot(), "SWTLW");
             if(swtlw.CmpNoCase("yes") == 0) {
-                LocalWorkspaceST::Get()->SetParserFlags(LocalWorkspaceST::Get()->GetParserFlags() |
-                                                        LocalWorkspace::EnableSWTLW);
+                GetLocalWorkspace()->SetParserFlags(GetLocalWorkspace()->GetParserFlags() |
+                                                    LocalWorkspace::EnableSWTLW);
                 SyncToLocalWorkspaceSTParserPaths();
                 SyncToLocalWorkspaceSTParserMacros();
             }
@@ -1307,16 +1309,12 @@ bool clCxxWorkspace::MoveProjectToFolder(const wxString& projectName, const wxSt
     wxXmlNode* folderXml = DoGetWorkspaceFolderXmlNode(folderPath);
     if(!folderXml) {
         folderXml = DoCreateWorkspaceFolder(folderPath);
-        if(!folderXml) {
-            return false;
-        }
+        if(!folderXml) { return false; }
     }
 
     // Locate the project XML node
     wxXmlNode* projectXml = DoGetProjectXmlNode(projectName);
-    if(!projectXml || !projectXml->GetParent()) {
-        return false;
-    }
+    if(!projectXml || !projectXml->GetParent()) { return false; }
 
     projectXml->GetParent()->RemoveChild(projectXml);
     folderXml->AddChild(projectXml);
@@ -1324,9 +1322,7 @@ bool clCxxWorkspace::MoveProjectToFolder(const wxString& projectName, const wxSt
     if(!saveAndReload) return true;
 
     // Store the XML file and reload the workspace
-    if(!SaveXmlFile()) {
-        return false;
-    }
+    if(!SaveXmlFile()) { return false; }
 
     // Reload the workspace XML file
     wxString errMsg;
@@ -1449,9 +1445,7 @@ void clCxxWorkspace::DoVisitWorkspaceFolders(wxXmlNode* parent, const wxString&
     while(child) {
         if(child->GetName() == "VirtualDirectory") {
             wxString tmppath = curpath;
-            if(!tmppath.IsEmpty()) {
-                tmppath << "/";
-            }
+            if(!tmppath.IsEmpty()) { tmppath << "/"; }
             tmppath << child->GetAttribute("Name", "");
             DoVisitWorkspaceFolders(child, tmppath, paths);
         }
@@ -1468,9 +1462,7 @@ wxString clCxxWorkspace::GetVersion() const
 wxFileName clCxxWorkspace::GetProjectFileName(const wxString& projectName) const
 {
     ProjectPtr p = GetProject(projectName);
-    if(!p) {
-        return wxFileName();
-    }
+    if(!p) { return wxFileName(); }
     return p->GetFileName();
 }
 
@@ -1491,11 +1483,48 @@ size_t clCxxWorkspace::GetExcludeFilesForConfig(std::vector<wxString>& files, co
             const wxStringSet_t& excludeFiles = proj->GetExcludeFiles();
             for(const wxString& filename : excludeFiles) {
                 clProjectFile::Ptr_t file = proj->GetFile(filename);
-                if(file && file->IsExcludeFromConfiguration(confname)) {
-                    files.push_back(filename);
-                }
+                if(file && file->IsExcludeFromConfiguration(confname)) { files.push_back(filename); }
             }
         }
     });
     return files.size();
 }
+
+void clCxxWorkspace::CreateCompileFlags() const
+{
+    // Build the global compiler paths, we will need this later on...
+    wxStringMap_t compilersGlobalPaths;
+    std::unordered_map<wxString, wxArrayString> pathsMap = BuildSettingsConfigST::Get()->GetCompilersGlobalPaths();
+    for(const std::unordered_map<wxString, wxArrayString>::value_type& vt : pathsMap) {
+        wxString compiler_name = vt.first;
+        wxArrayString pathsArr = vt.second;
+        wxString paths;
+        std::for_each(pathsArr.begin(), pathsArr.end(), [&](wxString& path) {
+            path.Trim().Trim(false);
+            if(path.EndsWith("\\")) { path.RemoveLast(); }
+            paths << path << ";";
+        });
+        compilersGlobalPaths.insert({ compiler_name, paths });
+    }
+
+    for(const ProjectMap_t::value_type& vt : m_projects) {
+        vt.second->CreateCompileFlags(compilersGlobalPaths);
+    }
+}
+
+void clCxxWorkspace::ClearBacktickCache() { m_backticks.clear(); }
+
+bool clCxxWorkspace::HasBacktick(const wxString& backtick) const { return m_backticks.count(backtick) != 0; }
+
+void clCxxWorkspace::SetBacktickValue(const wxString& backtick, const wxString& value)
+{
+    m_backticks.erase(backtick);
+    m_backticks.insert({ backtick, value });
+}
+
+bool clCxxWorkspace::GetBacktickValue(const wxString& backtick, wxString& value) const
+{
+    if(!HasBacktick(backtick)) { return false; }
+    value = m_backticks.find(backtick)->second;
+    return true;
+}
