@@ -30,10 +30,11 @@
 #include "SFTPTreeView.h"
 #include "SFTPUploadDialog.h"
 #include "SSHAccountManagerDlg.h"
-#include "SSHTerminal.h"
 #include "bitmap_loader.h"
 #include "clFileOrFolderDropTarget.h"
+#include "clToolBarButtonBase.h"
 #include "cl_config.h"
+#include "console_frame.h"
 #include "event_notifier.h"
 #include "fileutils.h"
 #include "globals.h"
@@ -50,6 +51,11 @@
 #include <wx/msgdlg.h>
 #include <wx/progdlg.h>
 #include <wx/textdlg.h>
+#include <wx/fdrepdlg.h>
+#include <wx/textdlg.h>
+#include "file_logger.h"
+#include "SFTPGrep.h"
+#include "SFTPStatusPage.h"
 
 static const int ID_NEW = ::wxNewId();
 static const int ID_RENAME = ::wxNewId();
@@ -68,8 +74,20 @@ SFTPTreeView::SFTPTreeView(wxWindow* parent, SFTP* plugin)
     , m_plugin(plugin)
 {
     m_bmpLoader = clGetManager()->GetStdIcons();
-    wxImageList* il = m_bmpLoader->MakeStandardMimeImageList();
-    m_treeCtrl->AssignImageList(il);
+    m_treeCtrl->SetBitmaps(m_bmpLoader->GetStandardMimeBitmapListPtr());
+
+    std::function<bool(const wxTreeItemId&, const wxTreeItemId&)> SortFunc = [&](const wxTreeItemId& itemA,
+                                                                                 const wxTreeItemId& itemB) {
+        MyClientData* a = static_cast<MyClientData*>(GetItemData(itemA));
+        MyClientData* b = static_cast<MyClientData*>(GetItemData(itemB));
+        if(a->IsFolder() && !b->IsFolder())
+            return true;
+        else if(b->IsFolder() && !a->IsFolder())
+            return false;
+        // same kind
+        return (a->GetFullName().CmpNoCase(b->GetFullName()) < 0);
+    };
+    m_treeCtrl->SetSortFunction(SortFunc);
 
     SFTPSettings settings;
     settings.Load();
@@ -85,7 +103,7 @@ SFTPTreeView::SFTPTreeView(wxWindow* parent, SFTP* plugin)
     m_treeCtrl->Connect(ID_NEW_FILE, wxEVT_MENU, wxCommandEventHandler(SFTPTreeView::OnMenuNewFile), NULL, this);
     m_treeCtrl->Connect(ID_REFRESH_FOLDER, wxEVT_MENU, wxCommandEventHandler(SFTPTreeView::OnMenuRefreshFolder), NULL,
                         this);
-
+    m_treeCtrl->Bind(wxEVT_MENU, &SFTPTreeView::OnExecuteCommand, this, ID_EXECUTE_COMMAND);
     wxTheApp->GetTopWindow()->Bind(wxEVT_MENU, &SFTPTreeView::OnCopy, this, wxID_COPY);
     wxTheApp->GetTopWindow()->Bind(wxEVT_MENU, &SFTPTreeView::OnCut, this, wxID_CUT);
     wxTheApp->GetTopWindow()->Bind(wxEVT_MENU, &SFTPTreeView::OnPaste, this, wxID_PASTE);
@@ -96,12 +114,31 @@ SFTPTreeView::SFTPTreeView(wxWindow* parent, SFTP* plugin)
     m_treeCtrl->SetDropTarget(new clFileOrFolderDropTarget(this));
     Bind(wxEVT_DND_FILE_DROPPED, &SFTPTreeView::OnFileDropped, this);
 
-    m_keyboardHelper.reset(new clTreeKeyboardInput(m_treeCtrl));
-    ::MSWSetNativeTheme(m_treeCtrl);
+    m_toolbar->AddTool(XRCID("ID_OPEN_ACCOUNT_MANAGER"), _("Open account manager..."),
+                       m_bmpLoader->LoadBitmap("folder-users"));
+    m_toolbar->AddTool(XRCID("ID_SFTP_CONNECT"), _("Disconnected. Click to connect"),
+                       m_bmpLoader->LoadBitmap("disconnected"));
+    m_toolbar->AddTool(XRCID("ID_ADD_BOOKMARK"), _("Add Bookmark"), m_bmpLoader->LoadBitmap("bookmark"), "",
+                       wxITEM_DROPDOWN);
+    m_toolbar->AddTool(XRCID("ID_SSH_OPEN_TERMINAL"), _("Open Terminal"), m_bmpLoader->LoadBitmap("console"), "",
+                       wxITEM_DROPDOWN);
+    m_toolbar->Realize();
+    // Bind the toolbar events
+    m_toolbar->Bind(wxEVT_TOOL, &SFTPTreeView::OnOpenAccountManager, this, XRCID("ID_OPEN_ACCOUNT_MANAGER"));
+    m_toolbar->Bind(wxEVT_TOOL, &SFTPTreeView::OnConnection, this, XRCID("ID_SFTP_CONNECT"));
+    m_toolbar->Bind(wxEVT_TOOL, &SFTPTreeView::OnAddBookmark, this, XRCID("ID_ADD_BOOKMARK"));
+    m_toolbar->Bind(wxEVT_UPDATE_UI, &SFTPTreeView::OnAddBookmarkUI, this, XRCID("ID_ADD_BOOKMARK"));
+    m_toolbar->Bind(wxEVT_TOOL_DROPDOWN, &SFTPTreeView::OnAddBookmarkMenu, this, XRCID("ID_ADD_BOOKMARK"));
+    m_toolbar->Bind(wxEVT_TOOL, &SFTPTreeView::OnOpenTerminal, this, XRCID("ID_SSH_OPEN_TERMINAL"));
+    m_toolbar->Bind(wxEVT_TOOL_DROPDOWN, &SFTPTreeView::OnOpenTerminalMenu, this, XRCID("ID_SSH_OPEN_TERMINAL"));
+    m_toolbar->Bind(wxEVT_UPDATE_UI, &SFTPTreeView::OnOpenTerminalUI, this, XRCID("ID_SSH_OPEN_TERMINAL"));
 }
 
 SFTPTreeView::~SFTPTreeView()
 {
+    if(m_channel && m_channel->IsOpen()) { m_channel->Close(); }
+    m_channel.reset(NULL);
+
     EventNotifier::Get()->Unbind(wxEVT_EDITOR_CLOSING, &SFTPTreeView::OnEditorClosing, this);
     wxTheApp->GetTopWindow()->Unbind(wxEVT_MENU, &SFTPTreeView::OnCopy, this, wxID_COPY);
     wxTheApp->GetTopWindow()->Unbind(wxEVT_MENU, &SFTPTreeView::OnCut, this, wxID_CUT);
@@ -136,8 +173,7 @@ void SFTPTreeView::DoBuildTree(const wxString& initialFolder)
     cd->SetIsFolder(true);
 
     wxTreeItemId root =
-        m_treeCtrl->AppendItem(m_treeCtrl->GetRootItem(), initialFolder,
-                               m_bmpLoader->GetMimeImageId(FileExtManager::TypeFolder), wxNOT_FOUND, cd);
+        m_treeCtrl->AddRoot(initialFolder, m_bmpLoader->GetMimeImageId(FileExtManager::TypeFolder), wxNOT_FOUND, cd);
 
     m_treeCtrl->AppendItem(root, "<dummy>");
     DoExpandItem(root);
@@ -152,9 +188,9 @@ void SFTPTreeView::OnItemActivated(wxTreeEvent& event)
     if(cd->IsFolder()) {
         wxTreeItemId item = event.GetItem();
         if(m_treeCtrl->IsExpanded(item)) {
-            m_treeCtrl->CallAfter(&wxTreeCtrl::Collapse, item);
+            m_treeCtrl->CallAfter(&clThemedTreeCtrl::Collapse, item);
         } else {
-            m_treeCtrl->CallAfter(&wxTreeCtrl::Expand, item);
+            m_treeCtrl->CallAfter(&clThemedTreeCtrl::Expand, item);
         }
 
     } else {
@@ -179,6 +215,9 @@ void SFTPTreeView::OnOpenAccountManager(wxCommandEvent& event)
 
 void SFTPTreeView::DoCloseSession()
 {
+    // Clear the 'search' view
+    m_plugin->GetOutputPane()->ClearSearchOutput();
+
     // Check if we have unmodified files belonged to this session
     // Load the session name
     IEditor::List_t editors;
@@ -317,9 +356,11 @@ void SFTPTreeView::OnContextMenu(wxContextMenuEvent& event)
             menu.AppendSeparator();
             menu.Append(ID_REFRESH_FOLDER, _("Refresh"));
             menu.AppendSeparator();
+            menu.Append(XRCID("sftp-find"), _("grep this folder..."));
+            menu.AppendSeparator();
         }
         menu.Append(ID_DELETE, _("Delete"));
-
+        menu.Bind(wxEVT_MENU, &SFTPTreeView::OnRemoteFind, this, XRCID("sftp-find"));
 #ifdef __WXMAC__
         menu.Enable(ID_DELETE, false);
 #endif
@@ -494,55 +535,51 @@ wxTreeItemId SFTPTreeView::DoAddFolder(const wxTreeItemId& parent, const wxStrin
     return wxTreeItemId();
 }
 
-void SFTPTreeView::OnAddBookmark(wxAuiToolBarEvent& event)
+void SFTPTreeView::OnAddBookmarkMenu(wxCommandEvent& event)
 {
-    if(event.IsDropDownClicked()) {
+    // Show the menu
+    const wxArrayString& bookmarks = m_account.GetBookmarks();
+    wxMenu menu;
+    for(size_t i = 0; i < bookmarks.GetCount(); ++i) {
+        menu.Append(ID_SFTP_BOOKMARK_FIRST + i, bookmarks.Item(i));
+    }
+    menu.AppendSeparator();
+    menu.Append(ID_SFTP_BOOKMARK_SETTINGS, _("Manage bookmarks..."));
 
-        // Show the menu
-        const wxArrayString& bookmarks = m_account.GetBookmarks();
-        wxMenu menu;
-        for(size_t i = 0; i < bookmarks.GetCount(); ++i) {
-            menu.Append(ID_SFTP_BOOKMARK_FIRST + i, bookmarks.Item(i));
-        }
-        menu.AppendSeparator();
-        menu.Append(ID_SFTP_BOOKMARK_SETTINGS, _("Manage bookmarks..."));
+    int sel = m_toolbar->GetMenuSelectionFromUser(XRCID("ID_SFTP_BOOKMARK_SETTINGS"), &menu);
+    if((sel >= ID_SFTP_BOOKMARK_FIRST) && (sel <= ID_SFTP_BOOKMARK_LAST)) {
+        // A bookmark was selected
+        CallAfter(&SFTPTreeView::DoBuildTree, bookmarks.Item(sel - ID_SFTP_BOOKMARK_FIRST));
 
-        wxPoint pt = event.GetItemRect().GetBottomLeft();
-        pt.y++;
+    } else if(sel == ID_SFTP_BOOKMARK_SETTINGS) {
+        // Bookmark settings
+        CallAfter(&SFTPTreeView::ManageBookmarks);
+    }
+}
 
-        int sel = m_auibar->GetPopupMenuSelectionFromUser(menu, pt);
-        if(sel >= ID_SFTP_BOOKMARK_FIRST && sel <= ID_SFTP_BOOKMARK_LAST) {
-            // A bookmark was selected
-            CallAfter(&SFTPTreeView::DoBuildTree, bookmarks.Item(sel - ID_SFTP_BOOKMARK_FIRST));
+void SFTPTreeView::OnAddBookmark(wxCommandEvent& event)
+{
+    try {
+        // sanity
+        if(!IsConnected()) return;
 
-        } else if(sel == ID_SFTP_BOOKMARK_SETTINGS) {
-            // Bookmark settings
-            CallAfter(&SFTPTreeView::ManageBookmarks);
-        }
+        // Get the current selection
+        MyClientDataVect_t selections = GetSelectionsItemData();
+        if(selections.size() != 1) return;
 
-    } else {
-        try {
-            // sanity
-            if(!IsConnected()) return;
+        MyClientData* cd = selections.at(0);
+        CHECK_PTR_RET(cd);
 
-            // Get the current selection
-            MyClientDataVect_t selections = GetSelectionsItemData();
-            if(selections.size() != 1) return;
+        if(!cd->IsFolder()) return;
 
-            MyClientData* cd = selections.at(0);
-            CHECK_PTR_RET(cd);
+        m_account.AddBookmark(cd->GetFullPath());
+        SFTPSettings settings;
+        settings.Load();
+        settings.UpdateAccount(m_account);
+        settings.Save();
 
-            if(!cd->IsFolder()) return;
-
-            m_account.AddBookmark(cd->GetFullPath());
-            SFTPSettings settings;
-            settings.Load();
-            settings.UpdateAccount(m_account);
-            settings.Save();
-
-        } catch(clException& e) {
-            ::wxMessageBox(e.What(), "SFTP", wxICON_ERROR | wxOK | wxCENTER);
-        }
+    } catch(clException& e) {
+        ::wxMessageBox(e.What(), "SFTP", wxICON_ERROR | wxOK | wxCENTER);
     }
 }
 
@@ -576,18 +613,20 @@ void SFTPTreeView::OnSelectionChanged(wxTreeEvent& event)
 void SFTPTreeView::OnConnection(wxCommandEvent& event)
 {
     SFTPImages images;
+    clToolBarButtonBase* button = m_toolbar->FindById(XRCID("ID_SFTP_CONNECT"));
+    CHECK_PTR_RET(button);
     if(IsConnected()) {
         // Disconnect
         DoCloseSession();
         // Update toobar image
-        m_auibar->FindTool(ID_SFTP_CONNECT)->SetBitmap(images.Bitmap("sftp_disconnected"));
-        m_auibar->FindTool(ID_SFTP_CONNECT)->SetShortHelp(_("Disconnected. Click to connect"));
+        button->SetBmp(m_bmpLoader->LoadBitmap("disconnected"));
+        button->SetLabel(_("Disconnected. Click to connect"));
     } else {
         DoOpenSession();
         if(IsConnected()) {
             // Update toobar image
-            m_auibar->FindTool(ID_SFTP_CONNECT)->SetBitmap(images.Bitmap("sftp_connected"));
-            m_auibar->FindTool(ID_SFTP_CONNECT)->SetShortHelp(_("Connected. Click to disconnect"));
+            button->SetBmp(m_bmpLoader->LoadBitmap("connected"));
+            button->SetLabel(_("Connected. Click to disconnect"));
         }
     }
 }
@@ -598,7 +637,7 @@ void SFTPTreeView::DoOpenSession()
     if(!GetAccountFromUser(m_account)) { return; }
 
     wxString message;
-    wxProgressDialog dlg(_("SFTP"), wxString(' ', 100) + "\n\n", 10);
+    wxProgressDialog dlg(_("SFTP"), wxString(' ', 100) + "\n\n", 10, EventNotifier::Get()->TopFrame());
     dlg.Show();
     dlg.Update(1, wxString() << _("Connecting to: ") << m_account.GetAccountName() << "..."
                              << _("\n(this may take a few seconds)"));
@@ -743,25 +782,28 @@ void SFTPTreeView::OnCut(wxCommandEvent& event)
     }
 }
 
-void SFTPTreeView::OnOpenTerminal(wxAuiToolBarEvent& event)
+void SFTPTreeView::OnOpenTerminalMenu(wxCommandEvent& event)
+{
+    wxMenu menu;
+    menu.Append(wxID_PREFERENCES);
+    menu.Bind(wxEVT_MENU, &SFTPTreeView::OnSftpSettings, this, wxID_PREFERENCES);
+    m_toolbar->ShowMenuForButton(XRCID("ID_SSH_OPEN_TERMINAL"), &menu);
+}
+
+void SFTPTreeView::OnOpenTerminal(wxCommandEvent& event)
 {
     // Open terminal to the selected account
-    if(event.IsDropDownClicked()) {
-        SFTPTreeViewBase::ShowAuiToolMenu(event);
+    SSHAccountInfo account;
+    if(!GetAccountFromUser(account)) { return; }
 
-    } else {
-        SSHAccountInfo account;
-        if(!GetAccountFromUser(account)) { return; }
+    wxString connectString;
+    connectString << account.GetUsername() << "@" << account.GetHost();
 
-        wxString connectString;
-        connectString << account.GetUsername() << "@" << account.GetHost();
+    SFTPSettings settings;
+    settings.Load();
 
-        SFTPSettings settings;
-        settings.Load();
-
-        const wxString& sshClient = settings.GetSshClient();
-        FileUtils::OpenSSHTerminal(sshClient, connectString, account.GetPassword(), account.GetPort());
-    }
+    const wxString& sshClient = settings.GetSshClient();
+    FileUtils::OpenSSHTerminal(sshClient, connectString, account.GetPassword(), account.GetPort());
 }
 
 void SFTPTreeView::OnOpenTerminalUI(wxUpdateUIEvent& event) { event.Enable(true); }
@@ -770,15 +812,7 @@ bool SFTPTreeView::DoOpenFile(const wxTreeItemId& item)
 {
     MyClientData* cd = GetItemData(item);
     if(!cd || cd->IsFolder()) { return false; }
-
-    RemoteFileInfo remoteFile;
-    remoteFile.SetAccount(m_account);
-    remoteFile.SetRemoteFile(cd->GetFullPath());
-
-    SFTPThreadRequet* req = new SFTPThreadRequet(remoteFile);
-    SFTPWorkerThread::Instance()->Add(req);
-
-    m_plugin->AddRemoteFile(remoteFile);
+    m_plugin->OpenFile(cd->GetFullPath());
     return true;
 }
 
@@ -941,3 +975,58 @@ void SFTPTreeView::DoLoadSession()
         }
     }
 }
+
+void SFTPTreeView::OnExecuteCommand(wxCommandEvent& event)
+{
+    wxArrayTreeItemIds items;
+    m_treeCtrl->GetSelections(items);
+    if(items.GetCount() != 1) return;
+
+    try {
+
+        MyClientData* cd = GetItemData(items.Item(0));
+        if(!cd) { return; } // ??
+        if(!m_sftp || !m_sftp->GetSsh()) { return; }
+        ConsoleFrame* frame = new ConsoleFrame(EventNotifier::Get()->TopFrame(), m_sftp->GetSsh());
+        frame->Show();
+
+    } catch(clException& e) {
+        ::wxMessageBox(e.What(), "SFTP", wxICON_ERROR | wxOK | wxCENTER);
+    }
+}
+
+void SFTPTreeView::OnRemoteFind(wxCommandEvent& event)
+{
+    if(!m_sftp || !m_sftp->GetSsh()) { return; }
+
+    wxArrayTreeItemIds items;
+    m_treeCtrl->GetSelections(items);
+    if(items.GetCount() != 1) { return; }
+    MyClientData* cd = GetItemData(items.Item(0));
+    if(!cd || !cd->IsFolder()) { return; }
+
+    wxString remoteFolder = cd->GetFullPath();
+
+    SFTPGrep grep(EventNotifier::Get()->TopFrame());
+    if(grep.ShowModal() != wxID_OK) { return; }
+
+    try {
+        if(m_channel && m_channel->IsOpen()) { m_channel->Close(); }
+        m_channel.reset(new clSSHChannel(m_sftp->GetSsh()));
+        m_channel->Open();
+
+        // Prepare the UI for new search
+        m_plugin->GetOutputPane()->ClearSearchOutput();
+        m_plugin->GetOutputPane()->ShowSearchTab();
+        clGetManager()->ShowOutputPane(_("SFTP Log"));
+
+        // Run the search
+        GrepData gd = grep.GetData();
+        wxString command = gd.GetGrepCommand(remoteFolder);
+        m_plugin->GetOutputPane()->AddSearchText(wxString() << "Running command: " << command);
+        m_channel->Execute(command, m_plugin->GetOutputPane());
+
+    } catch(clException& e) {
+        ::wxMessageBox(e.What(), "SFTP", wxICON_ERROR | wxOK | wxCENTER);
+    }
+}
