@@ -1,36 +1,36 @@
-#include "webtools.h"
-#include <wx/xrc/xmlres.h>
-#include "fileextmanager.h"
-#include <wx/stc/stc.h>
-#include "event_notifier.h"
-#include "codelite_events.h"
-#include "WebToolsSettings.h"
-#include <wx/menu.h>
-#include "NoteJSWorkspace.h"
-#include "NodeJSWorkspaceView.h"
-#include "clWorkspaceManager.h"
-#include "globals.h"
-#include "clWorkspaceView.h"
-#include <wx/dirdlg.h>
-#include "tags_options_data.h"
-#include "ctags_manager.h"
-#include "PhpLexerAPI.h"
-#include "PHPSourceFile.h"
-#include "NodeJSDebuggerPane.h"
-#include "WebToolsConfig.h"
-#include "fileutils.h"
+#include "NodeDebuggerPane.h"
 #include "NodeJSEvents.h"
+#include "NodeJSWorkspaceView.h"
+#include "NoteJSWorkspace.h"
+#include "PHPSourceFile.h"
+#include "PhpLexerAPI.h"
 #include "WebToolsBase.h"
+#include "WebToolsConfig.h"
+#include "WebToolsSettings.h"
 #include "bitmap_loader.h"
+#include "clWorkspaceManager.h"
+#include "clWorkspaceView.h"
+#include "codelite_events.h"
+#include "ctags_manager.h"
+#include "event_notifier.h"
+#include "file_logger.h"
+#include "fileextmanager.h"
+#include "fileutils.h"
+#include "globals.h"
+#include "tags_options_data.h"
+#include "webtools.h"
+#include <wx/dirdlg.h>
+#include <wx/menu.h>
+#include <wx/stc/stc.h>
+#include <wx/xrc/xmlres.h>
+#include "clNodeJS.h"
 
 static WebTools* thePlugin = NULL;
 
 // Define the plugin entry point
 CL_PLUGIN_API IPlugin* CreatePlugin(IManager* manager)
 {
-    if(thePlugin == 0) {
-        thePlugin = new WebTools(manager);
-    }
+    if(thePlugin == 0) { thePlugin = new WebTools(manager); }
     return thePlugin;
 }
 
@@ -50,19 +50,25 @@ WebTools::WebTools(IManager* manager)
     : IPlugin(manager)
     , m_lastColourUpdate(0)
     , m_clangOldFlag(false)
-    , m_nodejsDebuggerPane(NULL)
-    , m_hideToolBarOnDebugStop(false)
 {
     m_longName = _("Support for JavaScript, CSS/SCSS, HTML, XML and other web development tools");
     m_shortName = wxT("WebTools");
 
+    // Initialise NodeJS
+    WebToolsConfig& conf = WebToolsConfig::Get().Load();
+    wxFileName fnNodeJS(conf.GetNodejs());
+    wxArrayString hints;
+    if(fnNodeJS.FileExists()) { hints.Add(fnNodeJS.GetPath()); }
+    clNodeJS::Get().Initialise(hints);
+
+    // Update the configuration with the findings
+    conf.SetNodejs(clNodeJS::Get().GetNode().GetFullPath());
+    conf.SetNpm(clNodeJS::Get().GetNpm().GetFullPath());
+
     // Register our new workspace type
     NodeJSWorkspace::Get(); // Instantiate the singleton by faking a call
     clWorkspaceManager::Get().RegisterWorkspace(new NodeJSWorkspace(true));
 
-    WebToolsImages images;
-    // BitmapLoader::RegisterImage(FileExtManager::TypeWorkspaceNodeJS, images.Bitmap("m_bmpNodeJS"));
-
     // Create the syntax highligher worker thread
     m_jsColourThread = new JavaScriptSyntaxColourThread(this);
     m_jsColourThread->Create();
@@ -75,12 +81,6 @@ WebTools::WebTools(IManager* manager)
     // Context menu
     EventNotifier::Get()->Bind(wxEVT_CONTEXT_MENU_EDITOR, &WebTools::OnEditorContextMenu, this);
 
-    // Code completion related events
-    EventNotifier::Get()->Bind(wxEVT_CC_CODE_COMPLETE, &WebTools::OnCodeComplete, this);
-    EventNotifier::Get()->Bind(wxEVT_CC_CODE_COMPLETE_LANG_KEYWORD, &WebTools::OnCodeComplete, this);
-    EventNotifier::Get()->Bind(wxEVT_CC_CODE_COMPLETE_FUNCTION_CALLTIP, &WebTools::OnCodeCompleteFunctionCalltip, this);
-    EventNotifier::Get()->Bind(wxEVT_CC_FIND_SYMBOL, &WebTools::OnFindSymbol, this);
-
     // Workspace related events
     EventNotifier::Get()->Bind(wxEVT_WORKSPACE_CLOSED, &WebTools::OnWorkspaceClosed, this);
     EventNotifier::Get()->Bind(wxEVT_WORKSPACE_LOADED, &WebTools::OnWorkspaceLoaded, this);
@@ -89,16 +89,18 @@ WebTools::WebTools(IManager* manager)
     EventNotifier::Get()->Bind(wxEVT_ACTIVE_EDITOR_CHANGED, &WebTools::OnEditorChanged, this);
 
     // Debugger related
-    EventNotifier::Get()->Bind(wxEVT_NODEJS_DEBUGGER_STARTED, &WebTools::OnNodeJSDebuggerStarted, this);
+    EventNotifier::Get()->Bind(wxEVT_NODEJS_DEBUGGER_STARTED, &WebTools::OnNodeJSCliDebuggerStarted, this);
     EventNotifier::Get()->Bind(wxEVT_NODEJS_DEBUGGER_STOPPED, &WebTools::OnNodeJSDebuggerStopped, this);
     EventNotifier::Get()->Bind(wxEVT_DBG_IS_PLUGIN_DEBUGGER, &WebTools::OnIsDebugger, this);
 
     Bind(wxEVT_MENU, &WebTools::OnSettings, this, XRCID("webtools_settings"));
-    m_jsCodeComplete.Reset(new JSCodeCompletion(""));
-    m_xmlCodeComplete.Reset(new XMLCodeCompletion());
-    m_cssCodeComplete.Reset(new CSSCodeCompletion());
+    Bind(wxEVT_NODE_COMMAND_TERMINATED, &WebTools::OnNodeCommandCompleted, this);
+
+    m_jsCodeComplete.Reset(new JSCodeCompletion("", this));
+    m_xmlCodeComplete.Reset(new XMLCodeCompletion(this));
+    m_cssCodeComplete.Reset(new CSSCodeCompletion(this));
     m_jsctags.Reset(new clJSCTags());
-    
+
     // Connect the timer
     m_timer = new wxTimer(this);
     m_timer->Start(3000);
@@ -109,12 +111,7 @@ WebTools::WebTools(IManager* manager)
 
 WebTools::~WebTools() { NodeJSWorkspace::Free(); }
 
-clToolBar* WebTools::CreateToolBar(wxWindow* parent)
-{
-    // Create the toolbar to be used by the plugin
-    clToolBar* tb(NULL);
-    return tb;
-}
+void WebTools::CreateToolBar(clToolBar* toolbar) { wxUnusedVar(toolbar); }
 
 void WebTools::CreatePluginMenu(wxMenu* pluginsMenu)
 {
@@ -127,18 +124,24 @@ void WebTools::CreatePluginMenu(wxMenu* pluginsMenu)
 
 void WebTools::UnPlug()
 {
+    // Store the configuration changes
+    WebToolsConfig::Get().SaveConfig();
+    Unbind(wxEVT_NODE_COMMAND_TERMINATED, &WebTools::OnNodeCommandCompleted, this);
+
+    // Stop the debugger
+    if(NodeJSWorkspace::Get()->IsOpen() && NodeJSWorkspace::Get()->GetDebugger() &&
+       NodeJSWorkspace::Get()->GetDebugger()->IsRunning()) {
+        NodeJSWorkspace::Get()->GetDebugger()->StopDebugger();
+    }
+
     EventNotifier::Get()->Unbind(wxEVT_CONTEXT_MENU_EDITOR, &WebTools::OnEditorContextMenu, this);
     EventNotifier::Get()->Unbind(wxEVT_FILE_LOADED, &WebTools::OnFileLoaded, this);
     EventNotifier::Get()->Unbind(wxEVT_FILE_SAVED, &WebTools::OnFileSaved, this);
     EventNotifier::Get()->Unbind(wxEVT_CL_THEME_CHANGED, &WebTools::OnThemeChanged, this);
-    EventNotifier::Get()->Unbind(wxEVT_CC_CODE_COMPLETE, &WebTools::OnCodeComplete, this);
-    EventNotifier::Get()->Unbind(wxEVT_CC_CODE_COMPLETE_LANG_KEYWORD, &WebTools::OnCodeComplete, this);
-    EventNotifier::Get()->Unbind(
-        wxEVT_CC_CODE_COMPLETE_FUNCTION_CALLTIP, &WebTools::OnCodeCompleteFunctionCalltip, this);
     EventNotifier::Get()->Unbind(wxEVT_WORKSPACE_CLOSED, &WebTools::OnWorkspaceClosed, this);
     EventNotifier::Get()->Unbind(wxEVT_WORKSPACE_LOADED, &WebTools::OnWorkspaceLoaded, this);
     EventNotifier::Get()->Unbind(wxEVT_ACTIVE_EDITOR_CHANGED, &WebTools::OnEditorChanged, this);
-    EventNotifier::Get()->Unbind(wxEVT_NODEJS_DEBUGGER_STARTED, &WebTools::OnNodeJSDebuggerStarted, this);
+    EventNotifier::Get()->Unbind(wxEVT_NODEJS_DEBUGGER_STARTED, &WebTools::OnNodeJSCliDebuggerStarted, this);
     EventNotifier::Get()->Unbind(wxEVT_NODEJS_DEBUGGER_STOPPED, &WebTools::OnNodeJSDebuggerStopped, this);
     EventNotifier::Get()->Unbind(wxEVT_DBG_IS_PLUGIN_DEBUGGER, &WebTools::OnIsDebugger, this);
 
@@ -157,9 +160,7 @@ void WebTools::UnPlug()
 
 void WebTools::DoRefreshColours(const wxString& filename)
 {
-    if(FileExtManager::GetType(filename) == FileExtManager::TypeJS) {
-        m_jsColourThread->QueueFile(filename);
-    }
+    if(FileExtManager::GetType(filename) == FileExtManager::TypeJS) { m_jsColourThread->QueueFile(filename); }
 }
 
 void WebTools::ColourJavaScript(const JavaScriptSyntaxColourThread::Reply& reply)
@@ -187,33 +188,6 @@ void WebTools::OnThemeChanged(wxCommandEvent& event)
     }
 }
 
-void WebTools::OnCodeComplete(clCodeCompletionEvent& event)
-{
-    event.Skip();
-    IEditor* editor = m_mgr->GetActiveEditor();
-    if(editor && m_jsCodeComplete && IsJavaScriptFile(editor)) {
-        event.Skip(false);
-        if(InsideJSComment(editor) || InsideJSString(editor)) {
-            // User the word completion plugin instead
-            m_jsCodeComplete->TriggerWordCompletion();
-        } else {
-            m_jsCodeComplete->CodeComplete(editor);
-        }
-    } else if(editor && m_xmlCodeComplete && editor->GetCtrl()->GetLexer() == wxSTC_LEX_XML) {
-        // an XML file
-        event.Skip(false);
-        m_xmlCodeComplete->XmlCodeComplete(editor);
-    } else if(editor && m_xmlCodeComplete && IsHTMLFile(editor)) {
-        // Html code completion
-        event.Skip(false);
-        m_xmlCodeComplete->HtmlCodeComplete(editor);
-    } else if(editor && m_cssCodeComplete && IsCSSFile(editor)) {
-        // CSS code completion
-        event.Skip(false);
-        m_cssCodeComplete->CssCodeComplete(editor);
-    }
-}
-
 bool WebTools::IsJavaScriptFile(const wxFileName& filename)
 {
     return FileExtManager::GetType(filename.GetFullName()) == FileExtManager::TypeJS;
@@ -224,16 +198,6 @@ bool WebTools::IsJavaScriptFile(const wxString& filename)
     return FileExtManager::GetType(filename) == FileExtManager::TypeJS;
 }
 
-void WebTools::OnCodeCompleteFunctionCalltip(clCodeCompletionEvent& event)
-{
-    event.Skip();
-    IEditor* editor = m_mgr->GetActiveEditor();
-    if(editor && m_jsCodeComplete && IsJavaScriptFile(editor) && !InsideJSComment(editor)) {
-        event.Skip(false);
-        m_jsCodeComplete->CodeComplete(editor);
-    }
-}
-
 void WebTools::OnWorkspaceClosed(wxCommandEvent& event) { event.Skip(); }
 
 void WebTools::OnEditorChanged(wxCommandEvent& event)
@@ -245,14 +209,16 @@ void WebTools::OnEditorChanged(wxCommandEvent& event)
 void WebTools::OnSettings(wxCommandEvent& event)
 {
     WebToolsSettings settings(m_mgr->GetTheApp()->GetTopWindow());
-    settings.ShowModal();
-    if(m_jsCodeComplete) {
-        m_jsCodeComplete->Reload();
-        m_jsCodeComplete->ClearFatalError();
-    }
-    if(m_xmlCodeComplete) {
-        m_xmlCodeComplete->Reload();
-        m_jsCodeComplete->ClearFatalError();
+    if(settings.ShowModal() == wxID_OK) {
+        NodeJSWorkspace::Get()->AllocateDebugger();
+        if(m_jsCodeComplete) {
+            m_jsCodeComplete->Reload();
+            m_jsCodeComplete->ClearFatalError();
+        }
+        if(m_xmlCodeComplete) {
+            m_xmlCodeComplete->Reload();
+            m_jsCodeComplete->ClearFatalError();
+        }
     }
 }
 
@@ -283,9 +249,7 @@ bool WebTools::IsJavaScriptFile(IEditor* editor)
     if(FileExtManager::IsPHPFile(editor->GetFileName())) {
         wxStyledTextCtrl* ctrl = editor->GetCtrl();
         int styleAtCurPos = ctrl->GetStyleAt(ctrl->GetCurrentPos());
-        if(styleAtCurPos >= wxSTC_HJ_START && styleAtCurPos <= wxSTC_HJA_REGEX) {
-            return true;
-        }
+        if(styleAtCurPos >= wxSTC_HJ_START && styleAtCurPos <= wxSTC_HJA_REGEX) { return true; }
     }
     return false;
 }
@@ -297,13 +261,13 @@ bool WebTools::InsideJSComment(IEditor* editor)
     if(FileExtManager::IsJavascriptFile(editor->GetFileName())) {
         // Use the Cxx macros
         return styleAtCurPos == wxSTC_C_COMMENT || styleAtCurPos == wxSTC_C_COMMENTLINE ||
-            styleAtCurPos == wxSTC_C_COMMENTDOC || styleAtCurPos == wxSTC_C_COMMENTLINEDOC ||
-            styleAtCurPos == wxSTC_C_COMMENTDOCKEYWORD || styleAtCurPos == wxSTC_C_COMMENTDOCKEYWORDERROR ||
-            styleAtCurPos == wxSTC_C_PREPROCESSORCOMMENT;
+               styleAtCurPos == wxSTC_C_COMMENTDOC || styleAtCurPos == wxSTC_C_COMMENTLINEDOC ||
+               styleAtCurPos == wxSTC_C_COMMENTDOCKEYWORD || styleAtCurPos == wxSTC_C_COMMENTDOCKEYWORDERROR ||
+               styleAtCurPos == wxSTC_C_PREPROCESSORCOMMENT;
     } else if(FileExtManager::IsPHPFile(editor->GetFileName())) {
         if(styleAtCurPos >= wxSTC_HJ_START && styleAtCurPos <= wxSTC_HJA_REGEX) {
             return styleAtCurPos == wxSTC_HJ_COMMENT || styleAtCurPos == wxSTC_HJ_COMMENTLINE ||
-                styleAtCurPos == wxSTC_HJ_COMMENTDOC;
+                   styleAtCurPos == wxSTC_HJ_COMMENTDOC;
         }
     }
     return false;
@@ -316,12 +280,12 @@ bool WebTools::InsideJSString(IEditor* editor)
     if(FileExtManager::IsJavascriptFile(editor->GetFileName())) {
         // Use the Cxx macros
         return styleAtCurPos == wxSTC_C_STRING || styleAtCurPos == wxSTC_C_CHARACTER ||
-            styleAtCurPos == wxSTC_C_STRINGEOL || styleAtCurPos == wxSTC_C_STRINGRAW ||
-            styleAtCurPos == wxSTC_C_HASHQUOTEDSTRING;
+               styleAtCurPos == wxSTC_C_STRINGEOL || styleAtCurPos == wxSTC_C_STRINGRAW ||
+               styleAtCurPos == wxSTC_C_HASHQUOTEDSTRING;
     } else if(FileExtManager::IsPHPFile(editor->GetFileName())) {
         if(styleAtCurPos >= wxSTC_HJ_START && styleAtCurPos <= wxSTC_HJA_REGEX) {
             return styleAtCurPos == wxSTC_HJ_DOUBLESTRING || styleAtCurPos == wxSTC_HJ_SINGLESTRING ||
-                styleAtCurPos == wxSTC_HJ_STRINGEOL;
+                   styleAtCurPos == wxSTC_HJ_STRINGEOL;
         }
     }
     return false;
@@ -370,46 +334,15 @@ bool WebTools::IsHTMLFile(IEditor* editor)
     return false;
 }
 
-void WebTools::OnNodeJSDebuggerStarted(clDebugEvent& event)
-{
-    event.Skip();
-    m_savePerspective = clGetManager()->GetDockingManager()->SavePerspective();
-
-    wxWindow* parent = m_mgr->GetDockingManager()->GetManagedWindow();
-    // Show the debugger pane
-    if(!m_nodejsDebuggerPane) {
-        m_nodejsDebuggerPane = new NodeJSDebuggerPane(parent);
-        clGetManager()->GetDockingManager()->AddPane(m_nodejsDebuggerPane, wxAuiPaneInfo()
-                                                                               .Layer(5)
-                                                                               .Name("nodejs_debugger")
-                                                                               .Caption("Node.js Debugger")
-                                                                               .CloseButton(false)
-                                                                               .MaximizeButton()
-                                                                               .Bottom()
-                                                                               .Position(0));
-    }
-
-    wxString layout;
-    wxFileName fnNodeJSLayout(clStandardPaths::Get().GetUserDataDir(), "nodejs.layout");
-    fnNodeJSLayout.AppendDir("config");
-    if(FileUtils::ReadFileContent(fnNodeJSLayout, layout)) {
-        m_mgr->GetDockingManager()->LoadPerspective(layout);
-    }
-    EnsureAuiPaneIsVisible("nodejs_debugger", true);
-
-    m_hideToolBarOnDebugStop = false;
-    if(!m_mgr->AllowToolbar()) {
-        // Using native toolbar
-        m_hideToolBarOnDebugStop = !m_mgr->IsToolBarShown();
-        m_mgr->ShowToolBar(true);
-    }
-}
-
 void WebTools::OnNodeJSDebuggerStopped(clDebugEvent& event)
 {
     event.Skip();
 
-    wxFileName fnNodeJSLayout(clStandardPaths::Get().GetUserDataDir(), "nodejs.layout");
+    clDEBUG1() << "Saving NodeJS debugger perspective";
+
+    wxString layoutFileName = "nodejs.layout";
+    if(event.GetEventType() == wxEVT_NODEJS_DEBUGGER_STOPPED) { layoutFileName = "nodejs_cli.layout"; }
+    wxFileName fnNodeJSLayout(clStandardPaths::Get().GetUserDataDir(), layoutFileName);
     fnNodeJSLayout.AppendDir("config");
     FileUtils::WriteFileContent(fnNodeJSLayout, m_mgr->GetDockingManager()->SavePerspective());
 
@@ -417,21 +350,13 @@ void WebTools::OnNodeJSDebuggerStopped(clDebugEvent& event)
         m_mgr->GetDockingManager()->LoadPerspective(m_savePerspective);
         m_savePerspective.clear();
     }
-
-    if(m_hideToolBarOnDebugStop) {
-        m_mgr->ShowToolBar(false);
-    }
 }
 
 void WebTools::EnsureAuiPaneIsVisible(const wxString& paneName, bool update)
 {
     wxAuiPaneInfo& pi = m_mgr->GetDockingManager()->GetPane(paneName);
-    if(pi.IsOk() && !pi.IsShown()) {
-        pi.Show();
-    }
-    if(update) {
-        m_mgr->GetDockingManager()->Update();
-    }
+    if(pi.IsOk() && !pi.IsShown()) { pi.Show(); }
+    if(update) { m_mgr->GetDockingManager()->Update(); }
 }
 
 void WebTools::OnWorkspaceLoaded(wxCommandEvent& event)
@@ -439,12 +364,17 @@ void WebTools::OnWorkspaceLoaded(wxCommandEvent& event)
     event.Skip();
     wxFileName workspaceFile = event.GetString();
     if(FileExtManager::GetType(workspaceFile.GetFullPath()) == FileExtManager::TypeWorkspaceNodeJS) {
-        m_jsCodeComplete.Reset(new JSCodeCompletion(workspaceFile.GetPath()));
+        // we need to do this in 2 steps: this is the because the service provider will auto register us on destuctions
+        // by NAME this means, that 'reset' will first call 'new' followed by 'delete' so we will end up to be removed
+        // from the list of service providers
+        m_jsCodeComplete.Reset(nullptr);
+        m_jsCodeComplete.Reset(new JSCodeCompletion(workspaceFile.GetPath(), this));
     } else {
         // For non NodeJS workspaces, create the .tern files under
         // the .codelite folder
         workspaceFile.AppendDir(".codelite");
-        m_jsCodeComplete.Reset(new JSCodeCompletion(workspaceFile.GetPath()));
+        m_jsCodeComplete.Reset(nullptr);
+        m_jsCodeComplete.Reset(new JSCodeCompletion(workspaceFile.GetPath(), this));
     }
 }
 
@@ -453,16 +383,6 @@ bool WebTools::IsCSSFile(IEditor* editor)
     return (FileExtManager::GetType(editor->GetFileName().GetFullName()) == FileExtManager::TypeCSS);
 }
 
-void WebTools::OnFindSymbol(clCodeCompletionEvent& event)
-{
-    event.Skip();
-    IEditor* editor = m_mgr->GetActiveEditor();
-    if(editor && m_jsCodeComplete && IsJavaScriptFile(editor) && !InsideJSComment(editor)) {
-        event.Skip(false);
-        m_jsCodeComplete->FindDefinition(editor);
-    }
-}
-
 void WebTools::OnFileLoaded(clCommandEvent& event)
 {
     event.Skip();
@@ -474,9 +394,11 @@ void WebTools::OnFileSaved(clCommandEvent& event)
     event.Skip();
     DoRefreshColours(event.GetFileName());
     IEditor* editor = m_mgr->GetActiveEditor();
-    if(editor && m_jsCodeComplete && IsJavaScriptFile(editor) && !InsideJSComment(editor)) {
-        // m_jsCodeComplete->ReparseFile(editor);
-        m_jsCodeComplete->ResetTern();
+    if(editor && IsJavaScriptFile(editor) && !InsideJSComment(editor)) {
+        if(m_jsCodeComplete) { m_jsCodeComplete->ResetTern(false); }
+        // Remove all compiler markers
+        editor->DelAllCompilerMarkers();
+        if(WebToolsConfig::Get().IsLintOnSave()) { clNodeJS::Get().LintFile(event.GetFileName()); }
     }
 }
 
@@ -494,3 +416,43 @@ void WebTools::OnIsDebugger(clDebugEvent& event)
     event.Skip(); // always call skip
     // event.GetStrings().Add("NodeJS Debugger");
 }
+
+void WebTools::OnNodeJSCliDebuggerStarted(clDebugEvent& event)
+{
+    event.Skip();
+    m_savePerspective = clGetManager()->GetDockingManager()->SavePerspective();
+
+    wxWindow* parent = m_mgr->GetDockingManager()->GetManagedWindow();
+    // Show the debugger pane
+    if(!m_nodejsCliDebuggerPane) {
+        m_nodejsCliDebuggerPane = new NodeDebuggerPane(parent);
+        // Let the pane process the 'startup' event as well
+        m_nodejsCliDebuggerPane->GetEventHandler()->ProcessEvent(event);
+        clGetManager()->GetDockingManager()->AddPane(m_nodejsCliDebuggerPane, wxAuiPaneInfo()
+                                                                                  .MinSize(wxSize(-1, 300))
+                                                                                  .Layer(5)
+                                                                                  .Name("nodejs_cli_debugger")
+                                                                                  .Caption("Node.js Debugger")
+                                                                                  .CloseButton(false)
+                                                                                  .MaximizeButton()
+                                                                                  .Bottom()
+                                                                                  .Position(0));
+    }
+
+    wxString layout;
+    wxFileName fnNodeJSLayout(clStandardPaths::Get().GetUserDataDir(), "nodejs_cli.layout");
+    fnNodeJSLayout.AppendDir("config");
+    if(FileUtils::ReadFileContent(fnNodeJSLayout, layout)) { m_mgr->GetDockingManager()->LoadPerspective(layout); }
+    EnsureAuiPaneIsVisible("nodejs_cli_debugger", true);
+}
+
+void WebTools::OnNodeCommandCompleted(clProcessEvent& event)
+{
+    event.Skip();
+    if(event.GetString() == "npm-install-tern") {
+        // tern installation completed, enable the code completion again
+        clGetManager()->SetStatusMessage("tern installed", 5);
+        WebToolsConfig::Get().EnableJavaScriptFlag(WebToolsConfig::kJSEnableCC, true);
+        if(m_jsCodeComplete) { m_jsCodeComplete->ResetTern(true); }
+    }
+}
